/*
    Logo Filter for VirtualDub -- Add a bitmap logo to a video 
	with selectable position, alpha blending, and transparent color.
	Copyright (C) 1999-2000 Donald A. Graft

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

	The author can be contacted at:
	Donald Graft
	neuron2@home.com.
*/

#include <windows.h>
#include <commctrl.h>
#include <stdio.h>
#include <crtdbg.h>
#include <math.h>
#include <stdlib.h>

#include "ScriptInterpreter.h"
#include "ScriptError.h"
#include "ScriptValue.h"

#include "resource.h"
#include "filter.h"

///////////////////////////////////////////////////////////////////////////

int RunProc(const FilterActivation *fa, const FilterFunctions *ff);
int StartProc(FilterActivation *fa, const FilterFunctions *ff);
int EndProc(FilterActivation *fa, const FilterFunctions *ff);
long ParamProc(FilterActivation *fa, const FilterFunctions *ff);
int InitProc(FilterActivation *fa, const FilterFunctions *ff);
int ConfigProc(FilterActivation *fa, const FilterFunctions *ff, HWND hwnd);
void StringProc(const FilterActivation *fa, const FilterFunctions *ff, char *str);
void ScriptConfig(IScriptInterpreter *isi, void *lpVoid, CScriptValue *argv, int argc);
bool FssProc(FilterActivation *fa, const FilterFunctions *ff, char *buf, int buflen);
BOOL LoadBitmapFromBMPFile(LPTSTR szFileName, HBITMAP *phBitmap, HPALETTE *phPalette);


///////////////////////////////////////////////////////////////////////////

struct InterPixel {
	int x;
	int y;
	int d;
	int sum;
	unsigned char r,g,b;
};

struct InterPixelData {
	struct InterPixel nn;
	struct InterPixel nne;
	struct InterPixel ne;
	struct InterPixel ene;
	struct InterPixel ee;
	struct InterPixel ese;
	struct InterPixel se;
	struct InterPixel sse;
	struct InterPixel ss;
	struct InterPixel ssw;
	struct InterPixel sw;
	struct InterPixel wsw;
	struct InterPixel ww;
	struct InterPixel wnw;
	struct InterPixel nw;
	struct InterPixel nnw;
	int totald;
};





typedef struct MyFilterData {
	IFilterPreview		*ifp;
	unsigned char		*bitmap, *bits;
	long				bwidth, bheight, bpitch;
	unsigned int		depth;
	char				filename[1024];
	int					animate;
	int					x, y, alpha;
	int                 transparent, xr, xg, xb, tolerance;
	int					nextBMP;
	unsigned long		start, count, loop_count;
	unsigned long		fadeinlen, fadeoutend, fadeoutlen;
	unsigned long		duration, loops;
	struct InterPixelData *interpolation_data;
	int					*scalar_table;
} MyFilterData;


bool FssProc(FilterActivation *fa, const FilterFunctions *ff, char *buf, int buflen)
{
	MyFilterData *mfd = (MyFilterData *)fa->filter_data;
    char save[255];
	int i, j;

	i = j = 0;
	while(mfd->filename[i])
	{
		if (mfd->filename[i] == '\\')
		{
			save[j++] = '\\';
			save[j++] = '\\';
			i++;
		}
		else
		{
			save[j++] = mfd->filename[i++];
		}
	}
	save[j] = 0;
	_snprintf(buf, buflen, "Config(%d, %d, %d, %d, %d, %d, %d, %d, \"%s\", %d, %d, %d, %d, %d, %d, %d)",
		mfd->x,
		mfd->y,
		mfd->alpha,
		mfd->transparent,
		mfd->xr,
		mfd->xg,
		mfd->xb,
		mfd->tolerance,
		save,
		mfd->animate,
		mfd->start,
		mfd->duration,
		mfd->loops,
		mfd->fadeinlen,
		mfd->fadeoutend,
		mfd->fadeoutlen);

	return true;
}

void ScriptConfig(IScriptInterpreter *isi, void *lpVoid, CScriptValue *argv, int argc) {
	FilterActivation *fa = (FilterActivation *)lpVoid;
	MyFilterData *mfd = (MyFilterData *)fa->filter_data;

	mfd->x				= argv[0].asInt();
	mfd->y				= argv[1].asInt();
	mfd->alpha			= argv[2].asInt();
	mfd->transparent	= !!argv[3].asInt();
	mfd->xr				= argv[4].asInt();
	mfd->xg				= argv[5].asInt();
	mfd->xb				= argv[6].asInt();
	mfd->tolerance		= argv[7].asInt();
	strcpy(mfd->filename, *argv[8].asString());
	mfd->animate		= !!argv[9].asInt();
	mfd->start			= argv[10].asInt();
	mfd->duration		= argv[11].asInt();
	mfd->loops			= argv[12].asInt();
	mfd->fadeinlen		= argv[13].asInt();
	mfd->fadeoutend		= argv[14].asInt();
	mfd->fadeoutlen     = argv[15].asInt();
}

ScriptFunctionDef func_defs[]={
	{ (ScriptFunctionPtr)ScriptConfig, "Config", "0iiiiiiiisiiiiiii" },
	{ NULL },
};

CScriptObject script_obj={
	NULL, func_defs
};

struct FilterDefinition filterDef_tutorial = {

	NULL, NULL, NULL,		// next, prev, module
	"Delogo - Interpolate (alpha 1)",	// name
	"Interpolate logo area .",
							// desc
	"Leuf", 		// maker
	NULL,					// private_data
	sizeof(MyFilterData),	// inst_data_size

	InitProc,				// initProc
	NULL,					// deinitProc
	RunProc,				// runProc
	NULL,					// paramProc
	ConfigProc, 			// configProc
	StringProc, 			// stringProc
	StartProc,				// startProc
	EndProc,				// endProc

	&script_obj,			// script_obj
	FssProc,				// fssProc

};

void Load(MyFilterData *mfd)
{
	FILE *fp;
	char buf[6];
	DWORD size;
    char *p;
	int found;
	char path[255];

	/* Load bitmap and get pointer to bitmap bits. */
	mfd->depth = 0;
	strcpy(path, mfd->filename);
	p = path + strlen(path);
	while (p >= path && *p != '.') p--;
	if (*p != '.') return;
	if (mfd->animate)
	{
		found = 0;
		while (*(p-1) >= '0' && *(p-1) <= '9')
		{
			p--;
			found = 1;
			if (mfd->nextBMP == -1) mfd->nextBMP = 0;
		}
		if (found)
		{
			if (mfd->nextBMP != 0)
			{
				sprintf(p, "%04d.bmp", mfd->nextBMP);
			}
			mfd->nextBMP++;
		}
	}
	if ((fp = fopen(path, "rb")) == NULL)
	{
		if (mfd->animate && mfd->nextBMP >= 0)
		{
			mfd->count = 1;
			mfd->nextBMP = -1;
			Load(mfd);
			++mfd->loop_count;
		}
		return;
	}
	fread(buf, 1, 2, fp);
	if (buf[0] != 'B' || buf[1] != 'M') return;
	fread(&size, 1, 4, fp);
	fclose(fp);
	if (mfd->bitmap) free(mfd->bitmap);
	mfd->bitmap = (unsigned char *) malloc((size_t) size);
    if ((fp = fopen(path, "rb")) == NULL)
		return;
	fread(mfd->bitmap, 1, size, fp);
	fclose(fp);
	mfd->bits = mfd->bitmap + *((DWORD *)(mfd->bitmap + 10));
	mfd->bwidth = *((LONG *)(mfd->bitmap + 18));
	mfd->bheight = *((DWORD *)(mfd->bitmap + 22));
	mfd->bpitch = 3 * mfd->bwidth;
	if (mfd->bpitch & 0x3)
	{
		mfd->bpitch = (mfd->bpitch & 0xfffffffc) + 4;
	}
	mfd->depth = *((unsigned int *)(mfd->bitmap + 28));

	// Get background 

	int backr,backg,backb,currentr,currentg,currentb, x,y;

	unsigned char *bp;

	bp = mfd->bits;
	backb = *bp++;
	backg = *bp++;
	backr = *bp++;

	//initialize table

	mfd->scalar_table = (int*)malloc(sizeof(int)*mfd->bheight*mfd->bwidth);
	mfd->interpolation_data = (struct InterPixelData *)malloc(sizeof(InterPixelData)*mfd->bheight*mfd->bwidth);

	for (y = 0; y < mfd->bheight; y++)
	{
		bp = mfd->bits + y * mfd->bpitch;
		for (x = 0; x < mfd->bwidth; x++)
		{
			currentb = *bp++;
			currentg = *bp++;
			currentr = *bp++;
			if(currentb!=backb || currentg!=backg || currentr!=backr) {
				mfd->scalar_table[x+y*mfd->bwidth]=0;
			} else {
				mfd->scalar_table[x+y*mfd->bwidth]=1;
			}
		}
	}
	
	for (y = 1; y < mfd->bheight-1; y++) {
		for (x = 1; x < mfd->bwidth-1; x++) {
			int i = x+y*mfd->bwidth;
			if(mfd->scalar_table[i]==0) {
				int totald = 0;
				//find the nearest background pt in each 16 directions
				int dx,dy,cval,cd,highd=0;
				//north
				dy = 0;
				dx = 0;
				cval=0;
				while(cval==0) {
					dy--;
					cval = mfd->scalar_table[x+(y+dy)*mfd->bwidth];
				}
				mfd->interpolation_data[i].nn.x = dx;
				mfd->interpolation_data[i].nn.y = dy;
				cd = mfd->interpolation_data[i].nn.d = abs(dy);
				if(cd>highd) highd = cd;
				//east
				dy = 0;
				dx = 0;
				cval=0;
				while(cval==0) {
					dx++;
					cval = mfd->scalar_table[x+dx+y*mfd->bwidth];
				}
				mfd->interpolation_data[i].ee.x = dx;
				mfd->interpolation_data[i].ee.y = dy;
				cd = mfd->interpolation_data[i].ee.d = dx;
				if(cd>highd) highd = cd;
				//south
				dy = 0;
				dx = 0;
				cval=0;
				while(cval==0) {
					dy++;
					cval = mfd->scalar_table[x+(y+dy)*mfd->bwidth];
				}
				mfd->interpolation_data[i].ss.x = dx;
				mfd->interpolation_data[i].ss.y = dy;
				cd = mfd->interpolation_data[i].ss.d = dy;
				if(cd>highd) highd = cd;
				//west
				dy = 0;
				dx = 0;
				cval=0;
				while(cval==0) {
					dx--;
					cval = mfd->scalar_table[x+dx+y*mfd->bwidth];
				}
				mfd->interpolation_data[i].ww.x = dx;
				mfd->interpolation_data[i].ww.y = dy;
				cd = mfd->interpolation_data[i].ww.d = abs(dx);
				if(cd>highd) highd = cd;
				//northeast;
				dy = 0;
				dx = 0;
				cval=0;
				while(cval==0) {
					dy--;
					dx++;
					cval = mfd->scalar_table[x+dx+(y+dy)*mfd->bwidth];
				}
				mfd->interpolation_data[i].ne.x = dx;
				mfd->interpolation_data[i].ne.y = dy;
				cd = mfd->interpolation_data[i].ne.d = (int)(dx*1.5);
				if(cd>highd) highd = cd;
				//southeast;
				dy = 0;
				dx = 0;
				cval=0;
				while(cval==0) {
					dy++;
					dx++;
					cval = mfd->scalar_table[x+dx+(y+dy)*mfd->bwidth];
				}
				mfd->interpolation_data[i].se.x = dx;
				mfd->interpolation_data[i].se.y = dy;
				cd = mfd->interpolation_data[i].se.d = (int)(dy*1.5);
				if(cd>highd) highd = cd;
				//southwest;
				dy = 0;
				dx = 0;
				cval=0;
				while(cval==0) {
					dy++;
					dx--;
					cval = mfd->scalar_table[x+dx+(y+dy)*mfd->bwidth];
				}
				mfd->interpolation_data[i].sw.x = dx;
				mfd->interpolation_data[i].sw.y = dy;
				cd = mfd->interpolation_data[i].sw.d = (int)(dy*1.5);
				if(cd>highd) highd = cd;
				//northwest;
				dy = 0;
				dx = 0;
				cval=0;
				while(cval==0) {
					dy--;
					dx--;
					cval = mfd->scalar_table[x+dx+(y+dy)*mfd->bwidth];
				}
				mfd->interpolation_data[i].nw.x = dx;
				mfd->interpolation_data[i].nw.y = dy;
				cd = mfd->interpolation_data[i].nw.d = (int)(abs(dy)*1.5);
				if(cd>highd) highd = cd;

				//north-northeast
				dy = 0;
				dx = 0;
				cval=0;
				while(cval==0) {
					dy++;
					dx = (dy+1)/2;
					cval = mfd->scalar_table[x+dx+(y+dy)*mfd->bwidth];
				}
				mfd->interpolation_data[i].nne.x = dx;
				mfd->interpolation_data[i].nne.y = dy;
				cd = mfd->interpolation_data[i].nne.d = (int)(dx*2.5);
				if(cd>highd) highd = cd;

				//east-northeast
				dy = 0;
				dx = 0;
				cval=0;
				while(cval==0) {
					dx++;
					dy = (dx+1)/2;
					cval = mfd->scalar_table[x+dx+(y+dy)*mfd->bwidth];
				}
				mfd->interpolation_data[i].ene.x = dx;
				mfd->interpolation_data[i].ene.y = dy;
				cd = mfd->interpolation_data[i].ene.d = (int)(dy*2.5);
				if(cd>highd) highd = cd;

				//east-southeast
				dy = 0;
				dx = 0;
				cval=0;
				while(cval==0) {
					dx++;
					dy = -(dx+1)/2;
					cval = mfd->scalar_table[x+dx+(y+dy)*mfd->bwidth];
				}
				mfd->interpolation_data[i].ese.x = dx;
				mfd->interpolation_data[i].ese.y = dy;
				cd = mfd->interpolation_data[i].ese.d = (int)(abs(dy)*2.5);
				if(cd>highd) highd = cd;

				//south-southeast
				dy = 0;
				dx = 0;
				cval=0;
				while(cval==0) {
					dy--;
					dx = -(dy-1)/2;
					cval = mfd->scalar_table[x+dx+(y+dy)*mfd->bwidth];
				}
				mfd->interpolation_data[i].sse.x = dx;
				mfd->interpolation_data[i].sse.y = dy;
				cd = mfd->interpolation_data[i].sse.d = (int)(dx*2.5);
				if(cd>highd) highd = cd;

				//south-southwest
				dy = 0;
				dx = 0;
				cval=0;
				while(cval==0) {
					dy--;
					dx = (dy-1)/2;
					cval = mfd->scalar_table[x+dx+(y+dy)*mfd->bwidth];
				}
				mfd->interpolation_data[i].ssw.x = dx;
				mfd->interpolation_data[i].ssw.y = dy;
				cd = mfd->interpolation_data[i].ssw.d = (int)(abs(dx)*2.5);
				if(cd>highd) highd = cd;
				
				//west-southwest
				dy = 0;
				dx = 0;
				cval=0;
				while(cval==0) {
					dx--;
					dy = (dx-1)/2;
					cval = mfd->scalar_table[x+dx+(y+dy)*mfd->bwidth];
				}
				mfd->interpolation_data[i].wsw.x = dx;
				mfd->interpolation_data[i].wsw.y = dy;
				cd = mfd->interpolation_data[i].wsw.d = (int)(abs(dy)*2.5);
				if(cd>highd) highd = cd;

				//west-northwest
				dy = 0;
				dx = 0;
				cval=0;
				while(cval==0) {
					dx--;
					dy = -(dx-1)/2;
					cval = mfd->scalar_table[x+dx+(y+dy)*mfd->bwidth];
				}
				mfd->interpolation_data[i].wnw.x = dx;
				mfd->interpolation_data[i].wnw.y = dy;
				cd = mfd->interpolation_data[i].wnw.d = (int)(dy*2.5);
				if(cd>highd) highd = cd;
				
				//north-northwest
				dy = 0;
				dx = 0;
				cval=0;
				while(cval==0) {
					dy++;
					dx = -(dy+1)/2;
					cval = mfd->scalar_table[x+dx+(y+dy)*mfd->bwidth];
				}
				mfd->interpolation_data[i].nnw.x = dx;
				mfd->interpolation_data[i].nnw.y = dy;
				cd = mfd->interpolation_data[i].nnw.d = (int)(abs(dx)*2.5);
				if(cd>highd) highd = cd;
				
				
				
				totald += mfd->interpolation_data[i].nn.d = (highd - mfd->interpolation_data[i].nn.d + 1);			
				totald += mfd->interpolation_data[i].ss.d = (highd - mfd->interpolation_data[i].ss.d + 1);			
				totald += mfd->interpolation_data[i].ee.d = (highd - mfd->interpolation_data[i].ee.d + 1);			
				totald += mfd->interpolation_data[i].ww.d = (highd - mfd->interpolation_data[i].ww.d + 1);			
				totald += mfd->interpolation_data[i].ne.d = (highd - mfd->interpolation_data[i].ne.d + 1);			
				totald += mfd->interpolation_data[i].nw.d = (highd - mfd->interpolation_data[i].nw.d + 1);			
				totald += mfd->interpolation_data[i].se.d = (highd - mfd->interpolation_data[i].se.d + 1);			
				totald += mfd->interpolation_data[i].sw.d = (highd - mfd->interpolation_data[i].sw.d + 1);
				totald += mfd->interpolation_data[i].nne.d = (highd - mfd->interpolation_data[i].nne.d + 1);
				totald += mfd->interpolation_data[i].ene.d = (highd - mfd->interpolation_data[i].ene.d + 1);			
				totald += mfd->interpolation_data[i].ese.d = (highd - mfd->interpolation_data[i].ese.d + 1);			
				totald += mfd->interpolation_data[i].sse.d = (highd - mfd->interpolation_data[i].sse.d + 1);			
				totald += mfd->interpolation_data[i].ssw.d = (highd - mfd->interpolation_data[i].sse.d + 1);			
				totald += mfd->interpolation_data[i].wsw.d = (highd - mfd->interpolation_data[i].wsw.d + 1);			
				totald += mfd->interpolation_data[i].wnw.d = (highd - mfd->interpolation_data[i].wnw.d + 1);			
				totald += mfd->interpolation_data[i].nnw.d = (highd - mfd->interpolation_data[i].nnw.d + 1);
				mfd->interpolation_data[i].totald = totald;
				
			}
		}
	}
/////////////////////
//For Debugging Only
/////////////////////
	FILE* fileptr;

	fileptr = fopen("c:/scalars.txt","w");
	fprintf(fileptr,"%d %d %d\n",backr,backb,backg);

	for(y=0; y < mfd->bheight; y++) {
		for (x = 0; x < mfd->bwidth; x++) {
			fprintf(fileptr,"%4d\t",mfd->scalar_table[x+y*mfd->bwidth]);
		}
		fprintf(fileptr,"\n");
	}
	fclose(fileptr);

	fileptr = fopen("c:/totald.txt","w");

	for(y=0; y < mfd->bheight; y++) {
		for (x = 0; x < mfd->bwidth; x++) {
			fprintf(fileptr,"%4d\t",mfd->interpolation_data[x+y*mfd->bwidth].totald);
		}
		fprintf(fileptr,"\n");
	}
	fprintf(fileptr,"\n\n\n");
	for(y=0; y < mfd->bheight; y++) {
		for (x = 0; x < mfd->bwidth; x++) {
			int i = x+y*mfd->bwidth;
			fprintf(fileptr,"%4d\t",mfd->interpolation_data[i].nn.d);
			fprintf(fileptr,"%4d\t",mfd->interpolation_data[i].ne.d);
			fprintf(fileptr,"%4d\t",mfd->interpolation_data[i].ee.d);
			fprintf(fileptr,"%4d\t",mfd->interpolation_data[i].se.d);
			fprintf(fileptr,"%4d\t",mfd->interpolation_data[i].ss.d);
			fprintf(fileptr,"%4d\t",mfd->interpolation_data[i].sw.d);
			fprintf(fileptr,"%4d\t",mfd->interpolation_data[i].ww.d);
			fprintf(fileptr,"%4d\t",mfd->interpolation_data[i].nw.d);
			fprintf(fileptr,"\n");
		}
		fprintf(fileptr,"\n");
	}

	fclose(fileptr);
//////////////////////

}

int StartProc(FilterActivation *fa, const FilterFunctions *ff)
{
	MyFilterData *mfd = (MyFilterData *)fa->filter_data;

	/* Load bitmap and get pointer to bitmap bits. */
	Load(mfd);


	return 0;
}

int EndProc(FilterActivation *fa, const FilterFunctions *ff) {
	MyFilterData *mfd = (MyFilterData *)fa->filter_data;

	free(mfd->bitmap);
	mfd->bitmap = NULL;
	return 0;
}

int RunProc(const FilterActivation *fa, const FilterFunctions *ff) {
	MyFilterData *mfd = (MyFilterData *)fa->filter_data;
	const PixDim	width = fa->src.w;
	const PixDim	height = fa->src.h;
	Pixel32 *src, *dst;
	int x, y;
	unsigned long frame;
	unsigned long r, g, b;
	int X = mfd->x;
	int Y = mfd->y;
	double ALPHA = mfd->alpha / 255.0;
	double in = mfd->fadeinlen;
	double out = mfd->fadeoutlen;
	Pixel32 *CurrentRow;
	unsigned int max = mfd->fadeoutend;
	unsigned int min = mfd->fadeoutlen;

	// borrowing the alpha variable for blur
	int blurfactor = mfd->alpha;
	// borrowing the fade in for black
//	double black = (double)mfd->fadeinlen;


	int XR = mfd->xr;
	int XG = mfd->xg;
	int XB = mfd->xb;

	if (mfd->depth != 24) return(0);
	if (X > width - mfd->bwidth) X = width - mfd->bwidth;
	if (Y > height - mfd->bheight) Y = height - mfd->bheight;

	src = fa->src.data;
	dst = fa->dst.data;
	for (y = 0; y < height; y++)
	{
		for (x = 0; x < width; x++)
		{
			dst[x] = src[x];
		}
		src	= (Pixel *)((char *)src + fa->src.pitch);
		dst	= (Pixel *)((char *)dst + fa->dst.pitch);
	}

	
	frame = fa->pfsi->lCurrentFrame;
	if (frame == 0)
	{
		mfd->count = 0;
		mfd->loop_count = 0;
		mfd->nextBMP = -1;
		Load(mfd);
	}
    if (frame < mfd->start) return 0;
	if (mfd->nextBMP >= 0 && mfd->loops && mfd->loop_count >= mfd->loops) return 0;
	if ((mfd->nextBMP == -1) &&
		(mfd->duration && frame >= mfd->start + mfd->duration)) return 0;
	if ((mfd->nextBMP >= 0) &&
		(mfd->duration && ++mfd->count > mfd->duration))
	{
		mfd->count = 1;
		Load(mfd);
	}

	int by; //absolute bitmap y coord
	int bx; //absolute bitmap x coord
	// color values for 8 dirs
//	int rnn, rne, ree, rse, rss, rsw, rww, rnw;
//	int gnn, gne, gee, gse, gss, gsw, gww, gnw;
//	int bnn, bne, bee, bse, bss, bsw, bww, bnw;

	int bheight = mfd->bheight;
	int bwidth = mfd->bwidth;

	int *contrast_table = (int*)malloc(sizeof(int)*bheight*bwidth);
	int maxcontrast = 0;
	int numpixels = 0;

	//first pass ... gather data, fill the contrast map

	y = height - Y +1 - mfd->bheight; // absolute frame y coord

	src	= (Pixel *)((char *)fa->src.data + y * fa->src.pitch);

	for (by = 1; by < bheight-1; by++) {
		for (x = X+1, bx = 1; x < X + bwidth-1; x++, bx++)	{
			r = ((src[x] & 0xff0000) >> 16);
			g = ((src[x] & 0xff00) >> 8);
			b = (src[x] & 0xff);

			int i = bx+by*bwidth;
			if(mfd->scalar_table[i]==0) {

				int nnd = mfd->interpolation_data[i].nn.d;
				int ned = mfd->interpolation_data[i].ne.d;
				int eed = mfd->interpolation_data[i].ee.d;
				int sed = mfd->interpolation_data[i].se.d;
				int ssd = mfd->interpolation_data[i].ss.d;
				int swd = mfd->interpolation_data[i].sw.d;
				int wwd = mfd->interpolation_data[i].ww.d;
				int nwd = mfd->interpolation_data[i].nw.d;
				int nned = mfd->interpolation_data[i].nne.d;
				int ened = mfd->interpolation_data[i].ene.d;
				int esed = mfd->interpolation_data[i].ese.d;
				int ssed = mfd->interpolation_data[i].sse.d;
				int sswd = mfd->interpolation_data[i].ssw.d;
				int wswd = mfd->interpolation_data[i].wsw.d;
				int wnwd = mfd->interpolation_data[i].wnw.d;
				int nnwd = mfd->interpolation_data[i].nnw.d;



				int nnsum, nesum, eesum, sesum, sssum, swsum, wwsum, nwsum,
					nnesum, enesum, esesum, ssesum, sswsum, wswsum, wnwsum, nnwsum;
				int colormean = 0;
//				int dmean = 0;
				int numpoints = 0;
				int rtemp,gtemp,btemp;

				// get north rgb
				if(nnd) {
					CurrentRow = (Pixel *)((char *)fa->src.data + (y + mfd->interpolation_data[i].nn.y) * fa->src.pitch);
					mfd->interpolation_data[i].nn.r = rtemp = ((CurrentRow[x] & 0xff0000) >> 16);
					mfd->interpolation_data[i].nn.g = gtemp = ((CurrentRow[x] & 0xff00) >> 8);
					mfd->interpolation_data[i].nn.b = btemp = (CurrentRow[x] & 0xff);
					colormean += mfd->interpolation_data[i].nn.sum = nnsum = rtemp + gtemp + btemp;
					numpoints++;
				}
				//east
				if(eed) {
					CurrentRow = (Pixel *)((char *)fa->src.data + y * fa->src.pitch);
					mfd->interpolation_data[i].ee.r = rtemp = ((CurrentRow[x + mfd->interpolation_data[i].ee.x] & 0xff0000) >> 16);
					mfd->interpolation_data[i].ee.g = gtemp = ((CurrentRow[x + mfd->interpolation_data[i].ee.x] & 0xff00) >> 8);
					mfd->interpolation_data[i].ee.b = btemp = (CurrentRow[x + mfd->interpolation_data[i].ee.x] & 0xff);
					colormean += mfd->interpolation_data[i].ee.sum = eesum = rtemp + gtemp + btemp;
					numpoints++;
				}
				// south
				if(ssd) {
					CurrentRow = (Pixel *)((char *)fa->src.data + (y + mfd->interpolation_data[i].ss.y) * fa->src.pitch);
					mfd->interpolation_data[i].ss.r = rtemp = ((CurrentRow[x] & 0xff0000) >> 16);
					mfd->interpolation_data[i].ss.g = gtemp = ((CurrentRow[x] & 0xff00) >> 8);
					mfd->interpolation_data[i].ss.b = btemp = (CurrentRow[x] & 0xff);
					colormean += mfd->interpolation_data[i].ss.sum = sssum = rtemp + gtemp + btemp;
					numpoints++;
				}
				//west
				if(wwd) {
					CurrentRow = (Pixel *)((char *)fa->src.data + y * fa->src.pitch);
					mfd->interpolation_data[i].ww.r = rtemp = ((CurrentRow[x + mfd->interpolation_data[i].ww.x] & 0xff0000) >> 16);
					mfd->interpolation_data[i].ww.g = rtemp = ((CurrentRow[x + mfd->interpolation_data[i].ww.x] & 0xff00) >> 8);
					mfd->interpolation_data[i].ww.b = rtemp = (CurrentRow[x + mfd->interpolation_data[i].ww.x] & 0xff);
					colormean += mfd->interpolation_data[i].ww.sum = wwsum = rtemp + gtemp + btemp;
					numpoints++;
				}
				//ne
				if(ned) {
					CurrentRow = (Pixel *)((char *)fa->src.data + (y + mfd->interpolation_data[i].ne.y) * fa->src.pitch);
					mfd->interpolation_data[i].ne.r = rtemp = ((CurrentRow[x + mfd->interpolation_data[i].ne.x] & 0xff0000) >> 16);
					mfd->interpolation_data[i].ne.g = gtemp = ((CurrentRow[x + mfd->interpolation_data[i].ne.x] & 0xff00) >> 8);
					mfd->interpolation_data[i].ne.b = btemp = (CurrentRow[x + mfd->interpolation_data[i].ne.x] & 0xff);
					colormean += mfd->interpolation_data[i].ne.sum = nesum = rtemp + gtemp + btemp;
					numpoints++;
				}
				//se
				if(sed) {
					CurrentRow = (Pixel *)((char *)fa->src.data + (y + mfd->interpolation_data[i].se.y) * fa->src.pitch);
					mfd->interpolation_data[i].se.r = rtemp = ((CurrentRow[x + mfd->interpolation_data[i].se.x] & 0xff0000) >> 16);
					mfd->interpolation_data[i].se.g = gtemp = ((CurrentRow[x + mfd->interpolation_data[i].se.x] & 0xff00) >> 8);
					mfd->interpolation_data[i].se.b = btemp = (CurrentRow[x + mfd->interpolation_data[i].se.x] & 0xff);
					colormean += mfd->interpolation_data[i].se.sum = sesum = rtemp + gtemp + btemp;
					numpoints++;
				}
				//sw
				if(swd) { 
					CurrentRow = (Pixel *)((char *)fa->src.data + (y + mfd->interpolation_data[i].sw.y) * fa->src.pitch);
					mfd->interpolation_data[i].sw.r = rtemp = ((CurrentRow[x + mfd->interpolation_data[i].sw.x] & 0xff0000) >> 16);
					mfd->interpolation_data[i].sw.g = gtemp = ((CurrentRow[x + mfd->interpolation_data[i].sw.x] & 0xff00) >> 8);
					mfd->interpolation_data[i].sw.b = btemp = (CurrentRow[x + mfd->interpolation_data[i].sw.x] & 0xff);
					colormean += mfd->interpolation_data[i].sw.sum = swsum = rtemp + gtemp + btemp;
					numpoints++;
				}
				//nw
				if(nwd) {
					CurrentRow = (Pixel *)((char *)fa->src.data + (y + mfd->interpolation_data[i].nw.y) * fa->src.pitch);
					mfd->interpolation_data[i].nw.r = rtemp = ((CurrentRow[x + mfd->interpolation_data[i].nw.x] & 0xff0000) >> 16);
					mfd->interpolation_data[i].nw.g = gtemp = ((CurrentRow[x + mfd->interpolation_data[i].nw.x] & 0xff00) >> 8);
					mfd->interpolation_data[i].nw.b = btemp = (CurrentRow[x + mfd->interpolation_data[i].nw.x] & 0xff);
					colormean += mfd->interpolation_data[i].nw.sum = nwsum = rtemp + gtemp + btemp;
					numpoints++;
				}
				//nne
				if(nned) {				
					CurrentRow = (Pixel *)((char *)fa->src.data + (y + mfd->interpolation_data[i].nne.y) * fa->src.pitch);
					mfd->interpolation_data[i].nne.r = rtemp = ((CurrentRow[x + mfd->interpolation_data[i].nne.x] & 0xff0000) >> 16);
					mfd->interpolation_data[i].nne.g = gtemp = ((CurrentRow[x + mfd->interpolation_data[i].nne.x] & 0xff00) >> 8);
					mfd->interpolation_data[i].nne.b = btemp = (CurrentRow[x + mfd->interpolation_data[i].nne.x] & 0xff);
					colormean += mfd->interpolation_data[i].nne.sum = nnesum = rtemp + gtemp + btemp;
					numpoints++;
				}
				//ene
				if(ened) {				
					CurrentRow = (Pixel *)((char *)fa->src.data + (y + mfd->interpolation_data[i].ene.y) * fa->src.pitch);
					mfd->interpolation_data[i].ene.r = rtemp = ((CurrentRow[x + mfd->interpolation_data[i].ene.x] & 0xff0000) >> 16);
					mfd->interpolation_data[i].ene.g = gtemp = ((CurrentRow[x + mfd->interpolation_data[i].ene.x] & 0xff00) >> 8);
					mfd->interpolation_data[i].ene.b = btemp = (CurrentRow[x + mfd->interpolation_data[i].ene.x] & 0xff);
					colormean += mfd->interpolation_data[i].ene.sum = enesum = rtemp + gtemp + btemp;
					numpoints++;
				}
				//ese
				if(esed) {				
					CurrentRow = (Pixel *)((char *)fa->src.data + (y + mfd->interpolation_data[i].ese.y) * fa->src.pitch);
					mfd->interpolation_data[i].ese.r = rtemp = ((CurrentRow[x + mfd->interpolation_data[i].ese.x] & 0xff0000) >> 16);
					mfd->interpolation_data[i].ese.g = gtemp = ((CurrentRow[x + mfd->interpolation_data[i].ese.x] & 0xff00) >> 8);
					mfd->interpolation_data[i].ese.b = btemp = (CurrentRow[x + mfd->interpolation_data[i].ese.x] & 0xff);
					colormean += mfd->interpolation_data[i].ese.sum = esesum = rtemp + gtemp + btemp;
					numpoints++;
				}
				//sse
				if(ssed) {				
					CurrentRow = (Pixel *)((char *)fa->src.data + (y + mfd->interpolation_data[i].sse.y) * fa->src.pitch);
					mfd->interpolation_data[i].sse.r = rtemp = ((CurrentRow[x + mfd->interpolation_data[i].sse.x] & 0xff0000) >> 16);
					mfd->interpolation_data[i].sse.g = gtemp = ((CurrentRow[x + mfd->interpolation_data[i].sse.x] & 0xff00) >> 8);
					mfd->interpolation_data[i].sse.b = btemp = (CurrentRow[x + mfd->interpolation_data[i].sse.x] & 0xff);
					colormean += mfd->interpolation_data[i].sse.sum = ssesum = rtemp + gtemp + btemp;
					numpoints++;
				}
				//ssw
				if(sswd) {				
					CurrentRow = (Pixel *)((char *)fa->src.data + (y + mfd->interpolation_data[i].ssw.y) * fa->src.pitch);
					mfd->interpolation_data[i].ssw.r = rtemp = ((CurrentRow[x + mfd->interpolation_data[i].ssw.x] & 0xff0000) >> 16);
					mfd->interpolation_data[i].ssw.g = gtemp = ((CurrentRow[x + mfd->interpolation_data[i].ssw.x] & 0xff00) >> 8);
					mfd->interpolation_data[i].ssw.b = btemp = (CurrentRow[x + mfd->interpolation_data[i].ssw.x] & 0xff);
					colormean += mfd->interpolation_data[i].ssw.sum = sswsum = rtemp + gtemp + btemp;
					numpoints++;
				}
				//wsw
				if(wswd) {				
					CurrentRow = (Pixel *)((char *)fa->src.data + (y + mfd->interpolation_data[i].wsw.y) * fa->src.pitch);
					mfd->interpolation_data[i].wsw.r = rtemp = ((CurrentRow[x + mfd->interpolation_data[i].wsw.x] & 0xff0000) >> 16);
					mfd->interpolation_data[i].wsw.g = gtemp = ((CurrentRow[x + mfd->interpolation_data[i].wsw.x] & 0xff00) >> 8);
					mfd->interpolation_data[i].wsw.b = btemp = (CurrentRow[x + mfd->interpolation_data[i].wsw.x] & 0xff);
					colormean += mfd->interpolation_data[i].wsw.sum = wswsum = rtemp + gtemp + btemp;
					numpoints++;
				}
				//wnw
				if(wnwd) {				
					CurrentRow = (Pixel *)((char *)fa->src.data + (y + mfd->interpolation_data[i].wnw.y) * fa->src.pitch);
					mfd->interpolation_data[i].wnw.r = rtemp = ((CurrentRow[x + mfd->interpolation_data[i].wnw.x] & 0xff0000) >> 16);
					mfd->interpolation_data[i].wnw.g = gtemp = ((CurrentRow[x + mfd->interpolation_data[i].wnw.x] & 0xff00) >> 8);
					mfd->interpolation_data[i].wnw.b = btemp = (CurrentRow[x + mfd->interpolation_data[i].wnw.x] & 0xff);
					colormean += mfd->interpolation_data[i].wnw.sum = wnwsum = rtemp + gtemp + btemp;
					numpoints++;
				}
				//nnw
				if(nnwd) {				
					CurrentRow = (Pixel *)((char *)fa->src.data + (y + mfd->interpolation_data[i].nnw.y) * fa->src.pitch);
					mfd->interpolation_data[i].nnw.r = rtemp = ((CurrentRow[x + mfd->interpolation_data[i].nnw.x] & 0xff0000) >> 16);
					mfd->interpolation_data[i].nnw.g = gtemp = ((CurrentRow[x + mfd->interpolation_data[i].nnw.x] & 0xff00) >> 8);
					mfd->interpolation_data[i].nnw.b = btemp = (CurrentRow[x + mfd->interpolation_data[i].nnw.x] & 0xff);
					colormean += mfd->interpolation_data[i].nnw.sum = nnwsum = rtemp + gtemp + btemp;
					numpoints++;
				}
				
				
				
				int contrastmean = 0;
				if(numpoints) {
					colormean /= numpoints;
//					dmean = totald/numpoints;

					int contrasttolerance = 220;
//					int nnc=0,nec=0,eec=0,sec=0,ssc=0,swc=0,wwc=0,nwc=0;
					if(nnd) {
						contrastmean += abs(nnsum-colormean);
					} 
					if(ned) {
						contrastmean += abs(nesum-colormean);
					}
					if(eed) {
						contrastmean += abs(eesum-colormean);
					}
					if(sed) { 
						contrastmean += abs(nnsum-colormean);
					}
					if(ssd) {
						contrastmean += abs(nnsum-colormean);
					}
					if(swd) {
						contrastmean += abs(nnsum-colormean);
					}
					if(wwd) {
						contrastmean += abs(nnsum-colormean);
					}
					if(nwd) {
						contrastmean += abs(nnsum-colormean);
					}
					if(nned) {
						contrastmean += abs(nnesum-colormean);
					}
					if(ened) {
						contrastmean += abs(nnesum-colormean);
					}
					if(esed) {
						contrastmean += abs(nnesum-colormean);
					}
					if(ssed) {
						contrastmean += abs(nnesum-colormean);
					}
					if(sswd) {
						contrastmean += abs(nnesum-colormean);
					}
					if(wswd) {
						contrastmean += abs(nnesum-colormean);
					}
					if(wnwd) {
						contrastmean += abs(nnesum-colormean);
					}
					if(nnwd) {
						contrastmean += abs(nnesum-colormean);
					}

					contrastmean /= numpoints;	

					contrast_table[i] = contrastmean;
					if(contrastmean>maxcontrast) maxcontrast = contrastmean;
					numpixels++;

				} else {
					contrast_table[bx+by*bwidth] = 0;
				}

			} else {
				contrast_table[bx+by*bwidth] = 0;
			}

		}
		src	= (Pixel *)((char *)src + fa->src.pitch);
		y++;
	}
	// scale the contrast map?

//	for(by=1; by<bheight-1; by++) {
//		for(bx=1; bx<bwidth-1; bx++) {
//
//		}
//	}

	y = height - Y +1 - mfd->bheight;
	src	= (Pixel *)((char *)fa->src.data + y * fa->src.pitch);
	dst	= (Pixel *)((char *)fa->dst.data + y * fa->dst.pitch);
	int i;
	for (by = 1; by < bheight-1; by++) {
		for (x = X+1, bx = 1; x < X + bwidth-1; x++, bx++)	{
			i = bx+by*bwidth;
			if(mfd->scalar_table[i]==0) {
				r = ((src[x] & 0xff0000) >> 16);
				g = ((src[x] & 0xff00) >> 8);
				b = (src[x] & 0xff);
				
				int nncost=0, necost=0, eecost=0, secost=0, sscost=0, swcost=0, wwcost=0, nwcost=0;
				int nnecost=0, enecost=0, esecost=0, ssecost=0, sswcost=0, wswcost=0, wnwcost=0, nnwcost=0;

				int totalcost = 0;
				int currentval;
				int oldval;
				int dx,dy;
				
				// travel each dir and calculate "cost" for each dir
/*				
				// nn
				dx=0;
				dy=0;
				currentval = contrast_table[i];
				while(dy > mfd->interpolation_data[i].nn.y) {
					dy--;
					oldval = currentval;
					currentval = contrast_table[bx+(by+dy)*bwidth];
					nncost += abs(currentval-oldval);
				}
				// ee
				dy = 0;
				dx = 0;
				currentval = contrast_table[i];
				while(dx < mfd->interpolation_data[i].ee.x) {
					dx++;
					oldval = currentval;
					currentval = contrast_table[bx+dx+by*bwidth];
					eecost += abs(currentval-oldval);
				}
				// ss
				dy = 0;
				dx = 0;
				currentval = contrast_table[i];
				while(dy < mfd->interpolation_data[i].ss.y) {
					dy++;
					oldval = currentval;
					currentval = contrast_table[bx+(by+dy)*bwidth];
					sscost += abs(currentval-oldval);
				}
				// ww
				dy = 0;
				dx = 0;
				currentval = contrast_table[i];
				while(dx > mfd->interpolation_data[i].ww.x) {
					dx--;
					oldval = currentval;
					currentval = contrast_table[bx+dx+by*bwidth];
					wwcost += abs(currentval-oldval);
				}
				//ne;
				dy = 0;
				dx = 0;
				currentval = contrast_table[i];
				while(dx < mfd->interpolation_data[i].ne.x) {
					dy--;
					dx++;
					oldval = currentval;
					currentval = contrast_table[bx+dx+(by+dy)*bwidth];
					necost += abs(currentval-oldval);
				}
				//se;
				dy = 0;
				dx = 0;
				currentval = contrast_table[i];
				while(dx < mfd->interpolation_data[i].se.x) {
					dy++;
					dx++;
					oldval = currentval;
					currentval = contrast_table[bx+dx+(by+dy)*bwidth];
					secost += abs(currentval-oldval);
				}
				//sw;
				dy = 0;
				dx = 0;
				currentval = contrast_table[i];
				while(dx > mfd->interpolation_data[i].sw.x) {
					dy++;
					dx--;
					oldval = currentval;
					currentval = contrast_table[bx+dx+(by+dy)*bwidth];
					swcost += abs(currentval-oldval);
				}
				//nw;
				dy = 0;
				dx = 0;
				currentval = contrast_table[i];
				while(dx > mfd->interpolation_data[i].nw.x) {
					dy--;
					dx--;
					oldval = currentval;
					currentval = contrast_table[bx+dx+(by+dy)*bwidth];
					nwcost += abs(currentval-oldval);
				}

				//north-northeast
				dy = 0;
				dx = 0;
				currentval = contrast_table[i];
				while(dx < mfd->interpolation_data[i].nne.x) {
					dy++;
					dx = (dy+1)/2;
					oldval = currentval;
					currentval = contrast_table[bx+dx+(by+dy)*bwidth];
					nnecost += abs(currentval-oldval);
				}

				//east-northeast
				dy = 0;
				dx = 0;
				currentval = contrast_table[i];
				while(dx < mfd->interpolation_data[i].ene.x) {
					dx++;
					dy = (dx+1)/2;
					oldval = currentval;
					currentval = contrast_table[bx+dx+(by+dy)*bwidth];
					enecost += abs(currentval-oldval);
				}

				//east-southeast
				dy = 0;
				dx = 0;
				currentval = contrast_table[i];
				while(dx < mfd->interpolation_data[i].ese.x) {
					dx++;
					dy = -(dx+1)/2;
					oldval = currentval;
					currentval = contrast_table[bx+dx+(by+dy)*bwidth];
					esecost += abs(currentval-oldval);
				}

				//south-southeast
				dy = 0;
				dx = 0;
				currentval = contrast_table[i];
				while(dx < mfd->interpolation_data[i].sse.x) {
					dy--;
					dx = -(dy-1)/2;
					oldval = currentval;
					currentval = contrast_table[bx+dx+(by+dy)*bwidth];
					ssecost += abs(currentval-oldval);
				}

				//south-southwest
				dy = 0;
				dx = 0;
				currentval = contrast_table[i];
				while(dx > mfd->interpolation_data[i].ssw.x) {
					dy--;
					dx = (dy-1)/2;
					oldval = currentval;
					currentval = contrast_table[bx+dx+(by+dy)*bwidth];
					sswcost += abs(currentval-oldval);
				}
				
				//west-southwest
				dy = 0;
				dx = 0;
				currentval = contrast_table[i];
				while(dx > mfd->interpolation_data[i].wsw.x) {
					dx--;
					dy = (dx-1)/2;
					oldval = currentval;
					currentval = contrast_table[bx+dx+(by+dy)*bwidth];
					wswcost += abs(currentval-oldval);
				}

				//west-northwest
				dy = 0;
				dx = 0;
				currentval = contrast_table[i];
				while(dx > mfd->interpolation_data[i].wnw.x) {
					dx--;
					dy = -(dx-1)/2;
					oldval = currentval;
					currentval = contrast_table[bx+dx+(by+dy)*bwidth];
					wnwcost += abs(currentval-oldval);
				}
				
				//north-northwest
				dy = 0;
				dx = 0;
				currentval = contrast_table[i];
				while(dx > mfd->interpolation_data[i].nnw.x) {
					dy++;
					dx = -(dy+1)/2;
					oldval = currentval;
					currentval = contrast_table[bx+dx+(by+dy)*bwidth];
					nnwcost += abs(currentval-oldval);
				}

				int highestcost = nncost;
				if(necost>highestcost) highestcost=necost;
				if(eecost>highestcost) highestcost=eecost;
				if(secost>highestcost) highestcost=secost;
				if(sscost>highestcost) highestcost=sscost;
				if(swcost>highestcost) highestcost=swcost;
				if(wwcost>highestcost) highestcost=wwcost;
				if(nwcost>highestcost) highestcost=nwcost;
				if(nnecost>highestcost) highestcost=nnecost;
				if(enecost>highestcost) highestcost=enecost;
				if(esecost>highestcost) highestcost=esecost;
				if(ssecost>highestcost) highestcost=ssecost;
				if(sswcost>highestcost) highestcost=sswcost;
				if(wswcost>highestcost) highestcost=wswcost;
				if(wnwcost>highestcost) highestcost=wnwcost;
				if(nnwcost>highestcost) highestcost=nnwcost;

				nncost = highestcost-nncost+1;
				necost = highestcost-necost+1;
				eecost = highestcost-eecost+1;
				secost = highestcost-secost+1;
				sscost = highestcost-sscost+1;
				swcost = highestcost-swcost+1;
				wwcost = highestcost-wwcost+1;
				nwcost = highestcost-nwcost+1;

				nnecost = highestcost-nnecost+1;
				enecost = highestcost-enecost+1;
				esecost = highestcost-esecost+1;
				ssecost = highestcost-ssecost+1;
				sswcost = highestcost-sswcost+1;
				wswcost = highestcost-wswcost+1;
				wnwcost = highestcost-wnwcost+1;
				nnwcost = highestcost-nnwcost+1;


				totalcost = nncost + necost + eecost + secost + sscost + swcost + wwcost + nwcost +
					nnecost + enecost + esecost + ssecost + sswcost + wswcost + wnwcost + nnwcost;
				int costmean = totalcost/16;
				
				int contrast_threshold = 25;
				bool highcontrastpair = false;
				if((abs(mfd->interpolation_data[i].nn.sum-mfd->interpolation_data[i].ss.sum) > contrast_threshold) && (nncost > costmean) && (sscost > costmean)) {
					highcontrastpair = true;
					nncost = 0;
					sscost = 0;
				}
				if((abs(mfd->interpolation_data[i].ee.sum-mfd->interpolation_data[i].ww.sum) > contrast_threshold) && (eecost > costmean) && (wwcost > costmean)) {
					highcontrastpair = true;
					eecost = 0;
					wwcost = 0;
				}
				if((abs(mfd->interpolation_data[i].ne.sum-mfd->interpolation_data[i].sw.sum) > contrast_threshold) && (necost > costmean) && (swcost > costmean)) {
					highcontrastpair = true;
					necost = 0;
					swcost = 0;
				}
				if((abs(mfd->interpolation_data[i].nw.sum-mfd->interpolation_data[i].se.sum) > contrast_threshold) && (nwcost > costmean) && (secost > costmean)) {
					highcontrastpair = true;
					nwcost = 0;
					secost = 0;
				}
				if((abs(mfd->interpolation_data[i].nne.sum-mfd->interpolation_data[i].ssw.sum) > contrast_threshold) && (nnecost > costmean) && (sswcost > costmean)) {
					highcontrastpair = true;
					nnecost = 0;
					sswcost = 0;
				}
				if((abs(mfd->interpolation_data[i].ene.sum-mfd->interpolation_data[i].wsw.sum) > contrast_threshold) && (enecost > costmean) && (wswcost > costmean)) {
					highcontrastpair = true;
					enecost = 0;
					wswcost = 0;
				}
				if((abs(mfd->interpolation_data[i].ese.sum-mfd->interpolation_data[i].wnw.sum) > contrast_threshold) && (esecost > costmean) && (wnwcost > costmean)) {
					highcontrastpair = true;
					esecost = 0;
					wnwcost = 0;
				}
				if((abs(mfd->interpolation_data[i].sse.sum-mfd->interpolation_data[i].nnw.sum) > contrast_threshold) && (ssecost > costmean) && (nnwcost > costmean)) {
					highcontrastpair = true;
					ssecost = 0;
					nnwcost = 0;
				}

				
				
				totalcost = nncost + necost + eecost + secost + sscost + swcost + wwcost + nwcost +
					nnecost + enecost + esecost + ssecost + sswcost + wswcost + wnwcost + nnwcost;



				if(totalcost) {
					r = (mfd->interpolation_data[i].nn.r*nncost +
							mfd->interpolation_data[i].ne.r*necost +
							mfd->interpolation_data[i].ee.r*eecost +
							mfd->interpolation_data[i].se.r*secost +
							mfd->interpolation_data[i].ss.r*sscost +
							mfd->interpolation_data[i].sw.r*swcost +
							mfd->interpolation_data[i].ww.r*wwcost +
							mfd->interpolation_data[i].nw.r*nwcost +
							mfd->interpolation_data[i].nne.r*nnecost +
							mfd->interpolation_data[i].ene.r*enecost +
							mfd->interpolation_data[i].ese.r*esecost +
							mfd->interpolation_data[i].sse.r*ssecost +
							mfd->interpolation_data[i].ssw.r*sswcost +
							mfd->interpolation_data[i].wsw.r*wswcost +
							mfd->interpolation_data[i].wnw.r*wnwcost +
							mfd->interpolation_data[i].nnw.r*nnwcost
							) / totalcost;
					g = (mfd->interpolation_data[i].nn.g*nncost +
							mfd->interpolation_data[i].ne.g*necost +
							mfd->interpolation_data[i].ee.g*eecost +
							mfd->interpolation_data[i].se.g*secost +
							mfd->interpolation_data[i].ss.g*sscost +
							mfd->interpolation_data[i].sw.g*swcost +
							mfd->interpolation_data[i].ww.g*wwcost +
							mfd->interpolation_data[i].nw.g*nwcost +
							mfd->interpolation_data[i].nne.g*nnecost +
							mfd->interpolation_data[i].ene.g*enecost +
							mfd->interpolation_data[i].ese.g*esecost +
							mfd->interpolation_data[i].sse.g*ssecost +
							mfd->interpolation_data[i].ssw.g*sswcost +
							mfd->interpolation_data[i].wsw.g*wswcost +
							mfd->interpolation_data[i].wnw.g*wnwcost +
							mfd->interpolation_data[i].nnw.g*nnwcost
							) / totalcost;
					b = (mfd->interpolation_data[i].nn.b*nncost +
							mfd->interpolation_data[i].ne.b*necost +
							mfd->interpolation_data[i].ee.b*eecost +
							mfd->interpolation_data[i].se.b*secost +
							mfd->interpolation_data[i].ss.b*sscost +
							mfd->interpolation_data[i].sw.b*swcost +
							mfd->interpolation_data[i].ww.b*wwcost +
							mfd->interpolation_data[i].nw.b*nwcost +
							mfd->interpolation_data[i].nne.b*nnecost +
							mfd->interpolation_data[i].ene.b*enecost +
							mfd->interpolation_data[i].ese.b*esecost +
							mfd->interpolation_data[i].sse.b*ssecost +
							mfd->interpolation_data[i].ssw.b*sswcost +
							mfd->interpolation_data[i].wsw.b*wswcost +
							mfd->interpolation_data[i].wnw.b*wnwcost +
							mfd->interpolation_data[i].nnw.b*nnwcost
							) / totalcost;
*/
				int totald = mfd->interpolation_data[i].totald;
				if(totald) {

						r = (mfd->interpolation_data[i].nn.r*mfd->interpolation_data[i].nn.d +
							mfd->interpolation_data[i].ne.r*mfd->interpolation_data[i].ne.d +
							mfd->interpolation_data[i].ee.r*mfd->interpolation_data[i].ee.d +
							mfd->interpolation_data[i].se.r*mfd->interpolation_data[i].se.d +
							mfd->interpolation_data[i].ss.r*mfd->interpolation_data[i].ss.d +
							mfd->interpolation_data[i].sw.r*mfd->interpolation_data[i].sw.d +
							mfd->interpolation_data[i].ww.r*mfd->interpolation_data[i].ww.d +
							mfd->interpolation_data[i].nw.r*mfd->interpolation_data[i].nw.d +
							mfd->interpolation_data[i].nne.r*mfd->interpolation_data[i].nne.d +
							mfd->interpolation_data[i].ene.r*mfd->interpolation_data[i].ene.d +
							mfd->interpolation_data[i].ese.r*mfd->interpolation_data[i].ese.d +
							mfd->interpolation_data[i].sse.r*mfd->interpolation_data[i].sse.d +
							mfd->interpolation_data[i].ssw.r*mfd->interpolation_data[i].ssw.d +
							mfd->interpolation_data[i].wsw.r*mfd->interpolation_data[i].wsw.d +
							mfd->interpolation_data[i].wnw.r*mfd->interpolation_data[i].wnw.d +
							mfd->interpolation_data[i].nnw.r*mfd->interpolation_data[i].nnw.d
							) / totald;
						g = (mfd->interpolation_data[i].nn.g*mfd->interpolation_data[i].nn.d +
							mfd->interpolation_data[i].ne.g*mfd->interpolation_data[i].ne.d +
							mfd->interpolation_data[i].ee.g*mfd->interpolation_data[i].ee.d +
							mfd->interpolation_data[i].se.g*mfd->interpolation_data[i].se.d +
							mfd->interpolation_data[i].ss.g*mfd->interpolation_data[i].ss.d +
							mfd->interpolation_data[i].sw.g*mfd->interpolation_data[i].sw.d +
							mfd->interpolation_data[i].ww.g*mfd->interpolation_data[i].ww.d +
							mfd->interpolation_data[i].nw.g*mfd->interpolation_data[i].nw.d +
							mfd->interpolation_data[i].nne.g*mfd->interpolation_data[i].nne.d +
							mfd->interpolation_data[i].ene.g*mfd->interpolation_data[i].ene.d +
							mfd->interpolation_data[i].ese.g*mfd->interpolation_data[i].ese.d +
							mfd->interpolation_data[i].sse.g*mfd->interpolation_data[i].sse.d +
							mfd->interpolation_data[i].ssw.g*mfd->interpolation_data[i].ssw.d +
							mfd->interpolation_data[i].wsw.g*mfd->interpolation_data[i].wsw.d +
							mfd->interpolation_data[i].wnw.g*mfd->interpolation_data[i].wnw.d +
							mfd->interpolation_data[i].nnw.g*mfd->interpolation_data[i].nnw.d
							) / totald;
						b = (mfd->interpolation_data[i].nn.b*mfd->interpolation_data[i].nn.d +
							mfd->interpolation_data[i].ne.b*mfd->interpolation_data[i].ne.d +
							mfd->interpolation_data[i].ee.b*mfd->interpolation_data[i].ee.d +
							mfd->interpolation_data[i].se.b*mfd->interpolation_data[i].se.d +
							mfd->interpolation_data[i].ss.b*mfd->interpolation_data[i].ss.d +
							mfd->interpolation_data[i].sw.b*mfd->interpolation_data[i].sw.d +
							mfd->interpolation_data[i].ww.b*mfd->interpolation_data[i].ww.d +
							mfd->interpolation_data[i].nw.b*mfd->interpolation_data[i].nw.d +
							mfd->interpolation_data[i].nne.b*mfd->interpolation_data[i].nne.d +
							mfd->interpolation_data[i].ene.b*mfd->interpolation_data[i].ene.d +
							mfd->interpolation_data[i].ese.b*mfd->interpolation_data[i].ese.d +
							mfd->interpolation_data[i].sse.b*mfd->interpolation_data[i].sse.d +
							mfd->interpolation_data[i].ssw.b*mfd->interpolation_data[i].ssw.d +
							mfd->interpolation_data[i].wsw.b*mfd->interpolation_data[i].wsw.d +
							mfd->interpolation_data[i].wnw.b*mfd->interpolation_data[i].wnw.d +
							mfd->interpolation_data[i].nnw.b*mfd->interpolation_data[i].nnw.d
							) / totald;

					if(r<min) r=min;
					if(r>max) r=max;
					if(g<min) g=min;
					if(g>max) g=max;
					if(b<min) b=min;
					if(b>max) b=max;
				//	if(highcontrastpair) {
				//		r=max;
				//		g=min;
				//		b=min;
				//	}
				} else {
					// don't weight anything
				}



				dst[x] = ((r << 16) | (g << 8) | (b));
			}			
		}
		src	= (Pixel *)((char *)src + fa->src.pitch);
		dst	= (Pixel *)((char *)dst + fa->dst.pitch);
		y++;
	}
	free(contrast_table);
	contrast_table = NULL;
	
	return 0;
}

extern "C" int __declspec(dllexport) __cdecl VirtualdubFilterModuleInit2(FilterModule *fm, const FilterFunctions *ff, int& vdfd_ver, int& vdfd_compat);
extern "C" void __declspec(dllexport) __cdecl VirtualdubFilterModuleDeinit(FilterModule *fm, const FilterFunctions *ff);

static FilterDefinition *fd_tutorial;

int __declspec(dllexport) __cdecl VirtualdubFilterModuleInit2(FilterModule *fm, const FilterFunctions *ff, int& vdfd_ver, int& vdfd_compat) {
	if (!(fd_tutorial = ff->addFilter(fm, &filterDef_tutorial, sizeof(FilterDefinition))))
		return 1;

	vdfd_ver = VIRTUALDUB_FILTERDEF_VERSION;
	vdfd_compat = VIRTUALDUB_FILTERDEF_COMPATIBLE;

	return 0;
}

void __declspec(dllexport) __cdecl VirtualdubFilterModuleDeinit(FilterModule *fm, const FilterFunctions *ff) {
	ff->removeFilter(fd_tutorial);
}

int InitProc(FilterActivation *fa, const FilterFunctions *ff)
{
	MyFilterData *mfd = (MyFilterData *)fa->filter_data;
   	char prog[256];
	char path[256];
	LPTSTR ptr;

	mfd->x = 0;
	mfd->y = 0;
	mfd->alpha = 8;
	mfd->transparent = 1;
	mfd->animate = 0;
	mfd->xr = 0;
	mfd->xg = 0;
	mfd->xb = 0;
	mfd->tolerance = 0;
	mfd->start = mfd->count = mfd->loop_count = 0;
	// black
	mfd->fadeinlen = 0;
	// max = mfd->fadeoutend
	mfd->fadeoutend = 255;
	// min = mfd->fadeoutlen
	mfd->fadeoutlen = 0;
	mfd->duration = 0;
	mfd->loops = 0;
	mfd->nextBMP = -1;

	GetModuleFileName(NULL, prog, 255);
	GetFullPathName(prog, 255, path, &ptr);
	*ptr = 0;
	strcat(path, "plugins\\Logo.bmp");
	strcpy(mfd->filename, path);
	return 0;
}

BOOL CALLBACK ConfigDlgProc(HWND hdlg, UINT msg, WPARAM wParam, LPARAM lParam) {
	MyFilterData *mfd = (MyFilterData *)GetWindowLong(hdlg, DWL_USER);

	switch(msg) {
		case WM_INITDIALOG:
			SetWindowLong(hdlg, DWL_USER, lParam);
			mfd = (MyFilterData *)lParam;

			CheckDlgButton(hdlg, IDC_ENABLE, mfd->transparent ? BST_CHECKED : BST_UNCHECKED);
			CheckDlgButton(hdlg, IDC_ANIMATE, mfd->animate ? BST_CHECKED : BST_UNCHECKED);
			SetWindowText(GetDlgItem(hdlg, IDC_FILENAME), mfd->filename);

			SendMessage(GetDlgItem(hdlg, IDC_XSLIDER), TBM_SETRANGE, (WPARAM)TRUE, MAKELONG(0, 800));
			SendMessage(GetDlgItem(hdlg, IDC_XSLIDER), TBM_SETPOS, (WPARAM)TRUE, mfd->x);
			SetDlgItemInt(hdlg, IDC_XOFF, mfd->x, FALSE);
			SendMessage(GetDlgItem(hdlg, IDC_YSLIDER), TBM_SETRANGE, (WPARAM)TRUE, MAKELONG(0, 600));
			SendMessage(GetDlgItem(hdlg, IDC_YSLIDER), TBM_SETPOS, (WPARAM)TRUE, mfd->y);
			SetDlgItemInt(hdlg, IDC_YOFF, mfd->y, FALSE);
			
			SetDlgItemInt(hdlg, IDC_RED, mfd->xr, FALSE);
			SetDlgItemInt(hdlg, IDC_ALPHA, mfd->alpha, FALSE);
			SetDlgItemInt(hdlg, IDC_GREEN, mfd->xg, FALSE);
			SetDlgItemInt(hdlg, IDC_BLUE, mfd->xb, FALSE);
			SetDlgItemInt(hdlg, IDC_TOLERANCE, mfd->tolerance, FALSE);
			SetDlgItemInt(hdlg, IDC_START, mfd->start, FALSE);
			SetDlgItemInt(hdlg, IDC_DURATION, mfd->duration, FALSE);
			SetDlgItemInt(hdlg, IDC_LOOPS, mfd->loops, FALSE);
			SetDlgItemInt(hdlg, IDC_FADEINLEN, mfd->fadeinlen, FALSE);
			SetDlgItemInt(hdlg, IDC_FADEOUTEND, mfd->fadeoutend, FALSE);
			SetDlgItemInt(hdlg, IDC_FADEOUTLEN, mfd->fadeoutlen, FALSE);
			mfd->ifp->InitButton(GetDlgItem(hdlg, IDPREVIEW));
			return TRUE;

		case WM_HSCROLL:
			if ((HWND) lParam == GetDlgItem(hdlg, IDC_XSLIDER))
			{
				int x = SendMessage(GetDlgItem(hdlg, IDC_XSLIDER), TBM_GETPOS, 0, 0);
				if (x != mfd->x)
				{
					mfd->x = x;
					SetDlgItemInt(hdlg, IDC_XOFF, mfd->x, FALSE);
					mfd->ifp->RedoFrame();
				}
			}
		case WM_VSCROLL:
			if ((HWND) lParam == GetDlgItem(hdlg, IDC_YSLIDER))
			{
				int y = SendMessage(GetDlgItem(hdlg, IDC_YSLIDER), TBM_GETPOS, 0, 0);
				if (y != mfd->y)
				{
					mfd->y = y;
					SetDlgItemInt(hdlg, IDC_YOFF, mfd->y, FALSE);
					mfd->ifp->RedoFrame();
				}
			}
			break;

		case WM_COMMAND:
			switch(LOWORD(wParam)) {

			case IDPREVIEW:
				mfd->ifp->Toggle(hdlg);
				break;

			case IDOK:
				EndDialog(hdlg, 0);
				return TRUE;
			case IDHELP:
				{
				char prog[256];
				char path[256];
				LPTSTR ptr;
				GetModuleFileName(NULL, prog, 255);
				GetFullPathName(prog, 255, path, &ptr);
				*ptr = 0;
				strcat(path, "plugins\\Delogo.txt");
				OutputDebugString(path);
				OutputDebugString("\n");
				strcpy(prog, "Notepad ");
				strcat(prog, path);
				WinExec(prog, SW_SHOW);
				return TRUE;
				}

			case IDC_BROWSE:
				{
				OPENFILENAME ofn;
				mfd->filename[0] = NULL;
				ofn.lStructSize = sizeof(OPENFILENAME);
				ofn.hwndOwner = hdlg;
				ofn.hInstance = NULL;
				ofn.lpTemplateName = NULL;
				ofn.lpstrFilter = "All Files\0*.*\0\0";
				ofn.lpstrCustomFilter = NULL;
				ofn.nMaxCustFilter = 0;
				ofn.nFilterIndex = 1;
				ofn.lpstrFile = mfd->filename;
				ofn.nMaxFile = 1024;
				ofn.lpstrFileTitle = NULL;
				ofn.nMaxFileTitle = 0;
				ofn.lpstrInitialDir = NULL;
				ofn.lpstrTitle = "Choose Logo Bitmap File";
				ofn.Flags = OFN_CREATEPROMPT;
				ofn.nFileOffset = 0;
				ofn.nFileExtension = 0;
				ofn.lpstrDefExt = NULL;
				ofn.lCustData = 0;
				ofn.lpfnHook = NULL;
				GetOpenFileName(&ofn);
				if (mfd->filename[0] != 0)
				{
					SetWindowText(GetDlgItem(hdlg, IDC_FILENAME), mfd->filename);
					Load(mfd);
					mfd->ifp->UndoSystem();
					mfd->ifp->RedoSystem();
				}
				break;
				}


			case IDC_ALPHA:
				mfd->alpha = GetDlgItemInt(hdlg, IDC_ALPHA, 0, FALSE);
				mfd->ifp->RedoFrame();
				break;
			case IDC_ENABLE:
				mfd->transparent = !!IsDlgButtonChecked(hdlg, IDC_ENABLE);
				mfd->ifp->RedoFrame();
				break;
			case IDC_ANIMATE:
				mfd->animate = !!IsDlgButtonChecked(hdlg, IDC_ANIMATE);
				mfd->ifp->RedoFrame();
				break;
			case IDC_RED:
				mfd->xr = GetDlgItemInt(hdlg, IDC_RED, 0, FALSE);
				mfd->ifp->RedoFrame();
				break;
			case IDC_GREEN:
				mfd->xg = GetDlgItemInt(hdlg, IDC_GREEN, 0, FALSE);
				mfd->ifp->RedoFrame();
				break;
			case IDC_BLUE:
				mfd->xb = GetDlgItemInt(hdlg, IDC_BLUE, 0, FALSE);
				mfd->ifp->RedoFrame();
				break;
			case IDC_TOLERANCE:
				mfd->tolerance = GetDlgItemInt(hdlg, IDC_TOLERANCE, 0, FALSE);
				mfd->ifp->RedoFrame();
				break;
			case IDC_START:
				mfd->start = GetDlgItemInt(hdlg, IDC_START, 0, FALSE);
				mfd->ifp->RedoFrame();
				break;
			case IDC_FADEINLEN:
				mfd->fadeinlen = GetDlgItemInt(hdlg, IDC_FADEINLEN, 0, FALSE);
				mfd->ifp->RedoFrame();
				break;
			case IDC_FADEOUTEND:
				mfd->fadeoutend = GetDlgItemInt(hdlg, IDC_FADEOUTEND, 0, FALSE);
				mfd->ifp->RedoFrame();
				break;
			case IDC_FADEOUTLEN:
				mfd->fadeoutlen = GetDlgItemInt(hdlg, IDC_FADEOUTLEN, 0, FALSE);
				mfd->ifp->RedoFrame();
				break;
			case IDC_DURATION:
				mfd->duration = GetDlgItemInt(hdlg, IDC_DURATION, 0, FALSE);
				mfd->ifp->RedoFrame();
				break;
			case IDC_LOOPS:
				mfd->loops = GetDlgItemInt(hdlg, IDC_LOOPS, 0, FALSE);
				mfd->ifp->RedoFrame();
				break;
			case IDC_XPLUS:
				if (mfd->x < 800)
				{
					mfd->x += 1;
					SetDlgItemInt(hdlg, IDC_XOFF, mfd->x, FALSE);
					SendMessage(GetDlgItem(hdlg, IDC_XSLIDER), TBM_SETPOS, (WPARAM)TRUE, mfd->x);
					mfd->ifp->RedoFrame();
				}
				break;
			case IDC_XMINUS:
				if (mfd->x > 0)
				{
					mfd->x -= 1;
					SetDlgItemInt(hdlg, IDC_XOFF, mfd->x, FALSE);
					SendMessage(GetDlgItem(hdlg, IDC_XSLIDER), TBM_SETPOS, (WPARAM)TRUE, mfd->x);
					mfd->ifp->RedoFrame();
				}
				break;
			case IDC_YPLUS:
				if (mfd->y < 600)
				{
					mfd->y += 1;
					SetDlgItemInt(hdlg, IDC_YOFF, mfd->y, FALSE);
					SendMessage(GetDlgItem(hdlg, IDC_YSLIDER), TBM_SETPOS, (WPARAM)TRUE, mfd->y);
					mfd->ifp->RedoFrame();
				}
				break;
			case IDC_YMINUS:
				if (mfd->y > 0)
				{
					mfd->y -= 1;
					SetDlgItemInt(hdlg, IDC_YOFF, mfd->y, FALSE);
					SendMessage(GetDlgItem(hdlg, IDC_YSLIDER), TBM_SETPOS, (WPARAM)TRUE, mfd->y);
					mfd->ifp->RedoFrame();
				}
				break;
			case IDCANCEL:
				EndDialog(hdlg, 1);
				return TRUE;
			}
			break;
	}

	return FALSE;
}

int ConfigProc(FilterActivation *fa, const FilterFunctions *ff, HWND hwnd)
{
	MyFilterData *mfd = (MyFilterData *)fa->filter_data;
	MyFilterData mfd_old = *mfd;
	int ret;

	mfd->ifp = fa->ifp;

	if (DialogBoxParam(fa->filter->module->hInstModule, MAKEINTRESOURCE(IDD_FILTER),
		hwnd, ConfigDlgProc, (LPARAM)mfd))
	{
		*mfd = mfd_old;
		ret = TRUE;
	}
    else
	{
		ret = FALSE;
	}
	return(ret);
}

void StringProc(const FilterActivation *fa, const FilterFunctions *ff, char *str)
{
	MyFilterData *mfd = (MyFilterData *)fa->filter_data;
}
