/*
	DeLogo - Interpolate filter for VirtualDub

	Based on LogoAway by Chris Wojdon.  Interpolate over an unwanted
	portion of a video, such as a logo or source problem.  Adaptively
	chooses the best direction to interpolate using a best-fit
	approach while minimizing dramatic gradients between unrelated
	parts of the frame.

	Code framework, dialog box and bitmap loading routines taken from
	Donald Graft's Logo filter.

	The author can be contacted at:
	Leuf
	legend65@yahoo.com
*/

/*
    Logo Filter for VirtualDub -- Add a bitmap logo to a video 
	with selectable position, alpha blending, and transparent color.
	Copyright (C) 1999-2000 Donald A. Graft

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

	The author can be contacted at:
	Donald Graft
	neuron2@home.com.
*/

#include <windows.h>
#include <commctrl.h>
#include <stdio.h>
#include <crtdbg.h>
#include <math.h>
#include <stdlib.h>
#include <time.h>

#include "ScriptInterpreter.h"
#include "ScriptError.h"
#include "ScriptValue.h"

#include "resource.h"
#include "filter.h"

#define MT_NONE  0
#define MT_WHITE 1
#define MT_COLOR 2

#define MI_NONE   0
#define MI_NORMAL 1
#define MI_USE_1  2
#define MI_EAT    3

int DEFAULT_X			=0;
int DEFAULT_Y			=0;
int DEFAULT_START		=0;
int DEFAULT_END			=0;
int DEFAULT_FADEIN		=0;
int DEFAULT_FADEOUT		=0;
int DEFAULT_ALPHA		=0;
int DEFAULT_T1			=10;
int DEFAULT_T2			=25;
int DEFAULT_T3			=40;
int DEFAULT_B1			=5;
int DEFAULT_B2			=5;
int DEFAULT_B3			=20;
int DEFAULT_BB			=10;
int DEFAULT_SIDE		=0;
int DEFAULT_SCENE		=10;
int DEFAULT_WHITE		=230;
int DEFAULT_FEATHER		=8;
int DEFAULT_ALPHA_MIN	=150;
int DEFAULT_ALPHA_T		=80;
int DEFAULT_NOISE_LVL	=2;

///////////////////////////////////////////////////////////////////////////

int RunProc(const FilterActivation *fa, const FilterFunctions *ff);
int StartProc(FilterActivation *fa, const FilterFunctions *ff);
int EndProc(FilterActivation *fa, const FilterFunctions *ff);
long ParamProc(FilterActivation *fa, const FilterFunctions *ff);
int InitProc(FilterActivation *fa, const FilterFunctions *ff);
int ConfigProc(FilterActivation *fa, const FilterFunctions *ff, HWND hwnd);
void StringProc(const FilterActivation *fa, const FilterFunctions *ff, char *str);
void ScriptConfig(IScriptInterpreter *isi, void *lpVoid, CScriptValue *argv, int argc);
bool FssProc(FilterActivation *fa, const FilterFunctions *ff, char *buf, int buflen);
//BOOL LoadBitmapFromBMPFile(LPTSTR szFileName, HBITMAP *phBitmap, HPALETTE *phPalette);


///////////////////////////////////////////////////////////////////////////

// Holds distance and memory offset values for a lookup pixel 
struct InterPixel {
	int d;
	int offset;
};

// InterPixels for each interpolation direction
// (They are double letters because at one point interpolation used to be 8 way)
struct InterPixelData {
	struct InterPixel nn;
	struct InterPixel ee;
	struct InterPixel ss;
	struct InterPixel ww;
};

// Simplified pixel data for dumb/lazy programmer
struct DecomposedRGB {
	unsigned long r,g,b;
};


typedef struct MyFilterData {
	IFilterPreview		*ifp;
	unsigned char		*bitmap, *bits, *bitmap2, *bits2, *bitmap3, *bits3;
	long				bwidth, bheight, bpitch;
	unsigned int		depth;
	char				filename[1024];
	int					debug;
	int					x, y, alpha;
	int                 interpolate_side;
	unsigned long		start;
	unsigned long		duration, fadein, fadeout;
	struct InterPixelData *interpolation_data;
	int					*scalar_table, *ns_table, *ew_table;
	struct DecomposedRGB	*logobuffer1, *logobuffer2; 
	struct DecomposedRGB *logobuffer3; 
	Pixel32				*Prevlogo;
	int					*scalar_table2, *scalar_table3;
	int					threshold1, threshold2, threshold3;
	unsigned int		blur1, blur2, blur3, blurbitmap;
	struct DecomposedRGB *PrevFrameBorder;
	int					PrevWeight;
	int					SceneThreshold, ScenePercent;
	int					NumPoints;
	int					back1r,back1g,back1b,back2r,back2g,back2b,back3r,back3g,back3b;
	int					ModeTransparent, ModeInterpolate;
	int					white;
	unsigned char		conversiontable[256][256];
	int					feather;
	int					bitmap_difficulty;
	int					noise_level;
	bool				PrevSceneChange;
	int					*logobuffer_eat; 

} MyFilterData;

int LoadBitmaps(MyFilterData *mfd);
int LoadInterpolationTables(MyFilterData *mfd, const FilterActivation *fa);
void BlurArea(MyFilterData *mfd, const FilterActivation *fa, struct DecomposedRGB *buf1, struct DecomposedRGB *buf2, int *blur_table, int passes);
int GetInterpolationPts(MyFilterData *mfd, int *table, struct InterPixelData *data, const FilterActivation *fa);

ScriptFunctionDef func_defs[]={
	{ (ScriptFunctionPtr)ScriptConfig, "Config", "0siiiiiiiiiiiiii" },
	{ NULL },
};

CScriptObject script_obj={
	NULL, func_defs
};

struct FilterDefinition filterDef_tutorial = {

	NULL, NULL, NULL,		// next, prev, module
	"X-Logo (0.5)",	// name
	"That logo has ceased to be!",
							// desc
	"Leuf", 				// maker
	NULL,					// private_data
	sizeof(MyFilterData),	// inst_data_size

	InitProc,				// initProc
	NULL,					// deinitProc
	RunProc,				// runProc
	NULL,					// paramProc
	ConfigProc, 			// configProc
	StringProc, 			// stringProc
	StartProc,				// startProc
	EndProc,				// endProc

	&script_obj,			// script_obj
	FssProc,				// fssProc

};

int InitProc(FilterActivation *fa, const FilterFunctions *ff)
{
	MyFilterData *mfd = (MyFilterData *)fa->filter_data;
   	char prog[256];
	char path[256];
	char path2[256];
	char name[256];
	int value;
	LPTSTR ptr;
	
	GetModuleFileName(NULL, prog, 255);
	GetFullPathName(prog, 255, path, &ptr);
	*ptr = 0;
	strcpy(path2, path);
	strcat(path2, "plugins\\xlogo\\Xlogo.ini");

	strcat(path, "plugins\\Logo.bmp");
	strcpy(mfd->filename, path);

	FILE* fileptr;

	if((fileptr = fopen(path2, "r"))!=NULL) {
		while(!feof(fileptr)) {
			fscanf(fileptr,"%s\t%d",name,&value);
			if(strcmp(name,"DEFAULT_X")==0)				DEFAULT_X=value;
			else if(strcmp(name,"DEFAULT_Y")==0)		DEFAULT_Y=value;
			else if(strcmp(name,"DEFAULT_START")==0)	DEFAULT_START=value;
			else if(strcmp(name,"DEFAULT_END")==0)		DEFAULT_END=value;
			else if(strcmp(name,"DEFAULT_FADEIN")==0)	DEFAULT_FADEIN=value;
			else if(strcmp(name,"DEFAULT_FADEOUT")==0)	DEFAULT_FADEOUT=value;
			else if(strcmp(name,"DEFAULT_ALPHA")==0)	DEFAULT_ALPHA=value;
			else if(strcmp(name,"DEFAULT_T1")==0)		DEFAULT_T1=value;
			else if(strcmp(name,"DEFAULT_T2")==0)		DEFAULT_T2=value;
			else if(strcmp(name,"DEFAULT_T3")==0)		DEFAULT_T3=value;
			else if(strcmp(name,"DEFAULT_B1")==0)		DEFAULT_B1=value;
			else if(strcmp(name,"DEFAULT_B2")==0)		DEFAULT_B2=value;
			else if(strcmp(name,"DEFAULT_B3")==0)		DEFAULT_B3=value;
			else if(strcmp(name,"DEFAULT_BB")==0)		DEFAULT_BB=value;
			else if(strcmp(name,"DEFAULT_SIDE")==0)		DEFAULT_SIDE=value;
			else if(strcmp(name,"DEFAULT_SCENE")==0)	DEFAULT_SCENE=value;
			else if(strcmp(name,"DEFAULT_WHITE")==0)	DEFAULT_WHITE=value;
			else if(strcmp(name,"DEFAULT_FEATHER")==0)	DEFAULT_FEATHER=value;
			else if(strcmp(name,"DEFAULT_ALPHA_M")==0)	DEFAULT_ALPHA_MIN=value;
			else if(strcmp(name,"DEFAULT_ALPHA_T")==0)	DEFAULT_ALPHA_T=value;
			else if(strcmp(name,"DEFAULT_NOISE_L")==0)	DEFAULT_NOISE_LVL=value;
		}
		fclose(fileptr);
	}
	mfd->x = DEFAULT_X;
	mfd->y = DEFAULT_Y;
	mfd->alpha = DEFAULT_ALPHA;
	mfd->debug = 0;
	mfd->fadeout = DEFAULT_FADEOUT;
	mfd->interpolate_side = DEFAULT_SIDE;
	mfd->start = DEFAULT_START;
	mfd->duration = DEFAULT_END;
	mfd->fadein = DEFAULT_FADEIN;
	mfd->threshold1 = DEFAULT_T1;
	mfd->threshold2 = DEFAULT_T2;
	mfd->threshold3 = DEFAULT_T3;
	mfd->blur1 = DEFAULT_B1;
	mfd->blur2 = DEFAULT_B2;
	mfd->blur3 = DEFAULT_B3;
	mfd->blurbitmap = DEFAULT_BB;
	mfd->ScenePercent = DEFAULT_SCENE;
	mfd->SceneThreshold = DEFAULT_T1;
	mfd->ModeInterpolate = MT_NONE;
	mfd->ModeTransparent = MI_NONE;
	mfd->white = DEFAULT_WHITE;
	mfd->feather = DEFAULT_FEATHER;
	mfd->bitmap_difficulty = 0;
	mfd->noise_level = DEFAULT_NOISE_LVL;
	return 0;
}

int StartProc(FilterActivation *fa, const FilterFunctions *ff)
{
	MyFilterData *mfd = (MyFilterData *)fa->filter_data;

	int result = LoadBitmaps(mfd);
	if (mfd->x > fa->src.w - mfd->bwidth) mfd->x = fa->src.w - mfd->bwidth;
	if (mfd->y > fa->src.h - mfd->bheight) mfd->y = fa->src.h - mfd->bheight;

	if(!result) result = LoadInterpolationTables(mfd,fa);
	srand(time(NULL));

	return result;
}

int EndProc(FilterActivation *fa, const FilterFunctions *ff) {
	MyFilterData *mfd = (MyFilterData *)fa->filter_data;
	if(mfd->bitmap!=NULL) {
		free(mfd->bitmap);
		mfd->bitmap = NULL;
	}
	if(mfd->bitmap2!=NULL) {
		free(mfd->bitmap2);
		mfd->bitmap2 = NULL;
	}
	if(mfd->bitmap3!=NULL) {
		free(mfd->bitmap3);
		mfd->bitmap3 = NULL;
	}
	if(mfd->interpolation_data!=NULL) {
		free(mfd->interpolation_data);
		mfd->interpolation_data = NULL;
	}
	if(mfd->scalar_table!=NULL) {
		free(mfd->scalar_table);
		mfd->scalar_table = NULL;
	}
	if(mfd->ns_table!=NULL) {
		free(mfd->ns_table);
		mfd->ns_table = NULL;
	}
	if(mfd->ew_table!=NULL) {
		free(mfd->ew_table);
		mfd->ew_table = NULL;
	}
	if(mfd->logobuffer1!=NULL) {
		free(mfd->logobuffer1);
		mfd->logobuffer1 = NULL;
	}
	if(mfd->logobuffer2!=NULL) {
		free(mfd->logobuffer2);
		mfd->logobuffer2 = NULL;
	}
	if(mfd->logobuffer3!=NULL) {
		free(mfd->logobuffer3);
		mfd->logobuffer3 = NULL;
	}
	if(mfd->Prevlogo!=NULL) {
		free(mfd->Prevlogo);
		mfd->Prevlogo = NULL;
	}
	if(mfd->scalar_table2!=NULL) {
		free(mfd->scalar_table2);
		mfd->scalar_table2 = NULL;
	}
	if(mfd->scalar_table3!=NULL) {
		free(mfd->scalar_table3);
		mfd->scalar_table3 = NULL;
	}

	return 0;
}

int RunProc(const FilterActivation *fa, const FilterFunctions *ff) {
	MyFilterData *mfd = (MyFilterData *)fa->filter_data;

	Pixel32 *src, *dst, *src_fixed, *dst_fixed;

	int x, y;
	const PixDim	width = fa->src.w;
	const PixDim	height = fa->src.h;
	src_fixed = fa->src.data;
	dst_fixed = fa->dst.data;
	long src_pitch = fa->src.pitch;
	long dst_pitch = fa->dst.pitch;

	//copy the src frame over
	src = fa->src.data;
	dst = fa->dst.data;
	for (y = 0; y < height; y++)
	{
		for (x = 0; x < width; x++)
		{
			dst[x] = src[x];
		}
		src	= (Pixel *)((char *)src + src_pitch);
		dst	= (Pixel *)((char *)dst + dst_pitch);
	}

	// bail if we aren't between the start and end frames
	unsigned long frame = fa->pfsi->lCurrentFrame;

    if (frame < mfd->start) return 0;
	if(mfd->duration && frame > mfd->duration) return 0;

	// General variables

	unsigned long r, g, b;
	int X = mfd->x;
	int Y = mfd->y;
	int bheight = mfd->bheight;
	int bwidth = mfd->bwidth;
	int bpitch = mfd->bpitch;
	int counter,i,j;
	int by; //absolute bitmap y coord
	int bx; //absolute bitmap x coord

	// start Transparent processing

	struct DecomposedRGB *logobuffer3 = mfd->logobuffer3;

	Pixel32 *Prevlogo = mfd->Prevlogo;

	int min = 0;
	int max = 255;
	unsigned char *bp1,*bp2;
	int rb1, gb1, bb1,rcalc, gcalc, bcalc;
	int back1b = mfd->back1b;
	int back1g = mfd->back1g;
	int back1r = mfd->back1r;

	bool SceneChange = false;
	
	if(mfd->ModeTransparent==MT_COLOR) {
		int back2b = mfd->back2b;
		int back2g = mfd->back2g;
		int back2r = mfd->back2r;
		int diff1r, diff1g, diff1b, diff2r, diff2g, diff2b, diffcr, diffcg, diffcb;
		int totalweightr, totalweightg, totalweightb;
		int rb2, gb2, bb2;

		y = height - Y - bheight; // absolute frame y coord
		dst	= (Pixel *)((char *)dst_fixed + y * dst_pitch);
		for (by = 0; by < bheight; by++) {

			bp1 = mfd->bits + by * bpitch;
			bp2 = mfd->bits2 + by * bpitch;
			for (x = X, bx = 0; x < X + bwidth; x++, bx++)	{
				i = bx+by*bwidth;

				r = ((dst[x] & 0xff0000) >> 16);
				g = ((dst[x] & 0xff00) >> 8);
				b = (dst[x] & 0xff);

				bb1 = *bp1++;
				gb1 = *bp1++;
				rb1 = *bp1++;
				bb2 = *bp2++;
				gb2 = *bp2++;
				rb2 = *bp2++;

				if (rb1!=back1r && gb1!=back1g && bb1!=back1b) {
					diff1r = rb1-back1r;
					diff1g = gb1-back1g;
					diff1b = bb1-back1b;
					diff2r = rb2-back2r;
					diff2g = gb2-back2g;
					diff2b = bb2-back2b;
					
					totalweightr = abs(r-rb1)+abs(r-rb2);
					totalweightg = abs(g-gb1)+abs(g-gb2);
					totalweightb = abs(b-bb1)+abs(b-bb2);
					
					double a = (double(r)-rb1)/(totalweightr);
					
					diffcr = int(diff1r*(1-a) + diff2r*(a));
					rcalc = r - diffcr;
					
					a = (double(g)-gb1)/(totalweightg);					
					
					diffcg = int(diff1g*(1-a) + diff2g*(a));
					gcalc = g - diffcg;
					
					a = (double(b)-bb1)/(totalweightb);					
					
					diffcb = int(diff1b*(1-a) + diff2b*(a));
					bcalc = b - diffcb;
					if (rcalc < min) rcalc = min;
					if (rcalc > max) rcalc = max;
					
					if (gcalc < min) gcalc = min;
					if (gcalc > max) gcalc = max;
					
					if (bcalc < min) bcalc = min;
					if (bcalc > max) bcalc = max;
					
					logobuffer3[i].r = rcalc;
					logobuffer3[i].g = gcalc;
					logobuffer3[i].b = bcalc;

//					dst[x] = ((rcalc << 16) | (gcalc << 8) | (bcalc));
				} else {
					logobuffer3[i].r = r;
					logobuffer3[i].g = g;
					logobuffer3[i].b = b;
				}
			}
			dst	= (Pixel *)((char *)dst + dst_pitch);
			y++;
		}
	} else if(mfd->ModeTransparent==MT_WHITE) {
		y = height - Y - bheight; // absolute frame y coord
		dst	= (Pixel *)((char *)dst_fixed + y * dst_pitch);
		for (by = 0; by < bheight; by++) {

			bp1 = mfd->bits + by * bpitch;
			bp2 = mfd->bits2 + by * bpitch;
			for (x = X, bx = 0; x < X + bwidth; x++, bx++)	{
				i = bx+by*bwidth;

				r = ((dst[x] & 0xff0000) >> 16);
				g = ((dst[x] & 0xff00) >> 8);
				b = (dst[x] & 0xff);

				bb1 = *bp1++;
				gb1 = *bp1++;
				rb1 = *bp1++;
				if (rb1!=back1r && gb1!=back1g && bb1!=back1b) {
					bcalc = mfd->conversiontable[b][bb1];
					gcalc = mfd->conversiontable[g][gb1];
					rcalc = mfd->conversiontable[r][rb1];
					logobuffer3[i].r = rcalc;
					logobuffer3[i].g = gcalc;
					logobuffer3[i].b = bcalc;
				} else {
					logobuffer3[i].r = r;
					logobuffer3[i].g = g;
					logobuffer3[i].b = b;
				}
			}
			dst	= (Pixel *)((char *)dst + dst_pitch);
			y++;
		}
	}


	// start Interpolation processing

	Pixel32 *CurrentRow;
	Pixel32 CPixel;

	unsigned int blur1 = mfd->blur2; // eliminated blur1 control due to script limits
	unsigned int blur2 = mfd->blur2;
	unsigned int blur3 = mfd->blur3;
	unsigned int blurbitmap = mfd->blurbitmap;
	double ALPHA = mfd->alpha / 255.0;
	struct InterPixelData *interpolation_data = mfd->interpolation_data;
	struct DecomposedRGB	*logobuffer1 = mfd->logobuffer1;
	struct DecomposedRGB	*logobuffer2 = mfd->logobuffer2;
	int *ns_table = mfd->ns_table;
	int *ew_table = mfd->ew_table;
	int *scalar_table = mfd->scalar_table;
	int *scalar_table2 = mfd->scalar_table2;
	int same_threshold = mfd->threshold1;
	int same_threshold2 = mfd->threshold2;
	int same_threshold3 = mfd->threshold3;
	int same_threshold4 = 10;
	unsigned int interpolate_side = mfd->interpolate_side;

	// color values 
	int rnn,ree,rss,rww;
	int gnn,gee,gss,gww;
	int bnn,bee,bss,bww;
	int rtemp, gtemp, btemp;
	int radj, gadj, badj;
	int p_radj, p_gadj, p_badj;
	int r1,r2,g1,g2,b1,b2;
	// weights
	int nnd,eed,ssd,wwd;

	int NumPoints = mfd->NumPoints;
	int NumPoints_current = 0;
	int ScenePercent = mfd->ScenePercent;
	int SceneThreshold = mfd->threshold1;
	int SceneChangeTotal = 0;
	struct DecomposedRGB *PrevFrameBorder = mfd->PrevFrameBorder;
	bool ns, ew;
	int ns_val, ew_val;
	int weight;
	int dx,dy,cval,offset;

//	unsigned long ravg=0, gavg=0, bavg=0;
	
	unsigned long diff_ns = 0, diff_ew = 0;

	FILE* fileptr; // for debug output

	if((frame-mfd->start<=mfd->fadein && mfd->fadein!=0) || (mfd->duration-frame <= mfd->fadeout && mfd->fadeout!=0)) ALPHA = 0.0;
	
	for(counter=0; counter<bheight*bwidth; counter++) {
		scalar_table2[counter] = scalar_table[counter];
	}


	if(ALPHA<1.0 && (mfd->ModeInterpolate==MI_NORMAL || mfd->ModeInterpolate==MI_USE_1)) {
		if(interpolate_side) {
			switch(interpolate_side) 
			{
			case 1:
				y = height - Y-1;
				dst = (Pixel *)((char *)dst_fixed + y * dst_pitch);
				r1 = ((dst[X] & 0xff0000) >> 16);
				g1 = ((dst[X] & 0xff00) >> 8);
				b1 = (dst[X] & 0xff);
				r2 = ((dst[X+bwidth-1] & 0xff0000) >> 16);
				g2 = ((dst[X+bwidth-1] & 0xff00) >> 8);
				b2 = (dst[X+bwidth-1] & 0xff);
				for (x = X+1, bx = 1; x < X + bwidth-1; x++, bx++)	{
					r = (r1*(bwidth-bx)+r2*bx)/bwidth;
					g = (g1*(bwidth-bx)+g2*bx)/bwidth;
					b = (b1*(bwidth-bx)+b2*bx)/bwidth;
					dst[x] = ((r << 16) | (g << 8) | (b));
				}
				break;
			case 2:
				y = height - Y-1;
				dst = (Pixel *)((char *)dst_fixed + y * dst_pitch);
				r1 = ((dst[X+bwidth-1] & 0xff0000) >> 16);
				g1 = ((dst[X+bwidth-1] & 0xff00) >> 8);
				b1 = (dst[X+bwidth-1] & 0xff);
				y = height - Y - bheight;
				dst = (Pixel *)((char *)dst_fixed + y * dst_pitch);
				r2 = ((dst[X+bwidth-1] & 0xff0000) >> 16);
				g2 = ((dst[X+bwidth-1] & 0xff00) >> 8);
				b2 = (dst[X+bwidth-1] & 0xff);
				
				for (y = height - Y - bheight+1, by = 1; y < height - Y-1; y++, by++)	{
					dst = (Pixel *)((char *)dst_fixed + y * dst_pitch);
					r = (r2*(bheight-by)+r1*by)/bheight;
					g = (g2*(bheight-by)+g1*by)/bheight;
					b = (b2*(bheight-by)+b1*by)/bheight;
					dst[X+bwidth-1] = ((r << 16) | (g << 8) | (b));
				}
				break;
			case 3:
				y = height - Y - bheight;
				dst = (Pixel *)((char *)dst_fixed + y * dst_pitch);
				r1 = ((dst[X] & 0xff0000) >> 16);
				g1 = ((dst[X] & 0xff00) >> 8);
				b1 = (dst[X] & 0xff);
				r2 = ((dst[X+bwidth-1] & 0xff0000) >> 16);
				g2 = ((dst[X+bwidth-1] & 0xff00) >> 8);
				b2 = (dst[X+bwidth-1] & 0xff);
				for (x = X+1, bx = 1; x < X + bwidth-1; x++, bx++)	{
					r = (r1*(bwidth-bx)+r2*bx)/bwidth;
					g = (g1*(bwidth-bx)+g2*bx)/bwidth;
					b = (b1*(bwidth-bx)+b2*bx)/bwidth;
					dst[x] = ((r << 16) | (g << 8) | (b));
				}
				break;
			case 4:
				y = height - Y-1;
				dst = (Pixel *)((char *)dst_fixed + y * dst_pitch);
				r1 = ((dst[X] & 0xff0000) >> 16);
				g1 = ((dst[X] & 0xff00) >> 8);
				b1 = (dst[X] & 0xff);
				y = height - Y - bheight;
				dst = (Pixel *)((char *)dst_fixed + y * dst_pitch);
				r2 = ((dst[X] & 0xff0000) >> 16);
				g2 = ((dst[X] & 0xff00) >> 8);
				b2 = (dst[X] & 0xff);
				for (y = height - Y - bheight+1, by = 1; y < height - Y-1; y++, by++)	{
					dst = (Pixel *)((char *)dst_fixed + y * dst_pitch);
					r = (r2*(bheight-by)+r1*by)/bheight;
					g = (g2*(bheight-by)+g1*by)/bheight;
					b = (b2*(bheight-by)+b1*by)/bheight;
					dst[X] = ((r << 16) | (g << 8) | (b));
				}
				break;
				
			}
		}
		
		y = height - Y +1 - bheight; // absolute frame y coord
		
		
		dst	= (Pixel *)((char *)dst_fixed + y * dst_pitch);
		for (by = 1; by < bheight-1; by++) {
			for (x = X+1, bx = 1; x < X + bwidth-1; x++, bx++)	{
				r = ((dst[x] & 0xff0000) >> 16);
				g = ((dst[x] & 0xff00) >> 8);
				b = (dst[x] & 0xff);
				
				i = bx+by*bwidth;
				ns_val = ns_table[i];
				ew_val = ew_table[i];
				ns = ns_val!=0;
				ew = ew_val!=0;
				
				if(ns || ew) {
					nnd = interpolation_data[i].nn.d;
					eed = interpolation_data[i].ee.d;
					ssd = interpolation_data[i].ss.d;
					wwd = interpolation_data[i].ww.d;
					
					CPixel = *(Pixel *)((char *)dst_fixed + interpolation_data[i].ee.offset);			
					ree = ((CPixel & 0xff0000) >> 16);
					gee = ((CPixel & 0xff00) >> 8);
					bee = (CPixel & 0xff);
					
					CPixel = *(Pixel *)((char *)dst_fixed + interpolation_data[i].ww.offset);			
					rww = ((CPixel & 0xff0000) >> 16);
					gww = ((CPixel & 0xff00) >> 8);
					bww = (CPixel & 0xff);
					
					CPixel = *(Pixel *)((char *)dst_fixed + interpolation_data[i].nn.offset);			
					rnn = ((CPixel & 0xff0000) >> 16);
					gnn = ((CPixel & 0xff00) >> 8);
					bnn = (CPixel & 0xff);
					
					CPixel = *(Pixel *)((char *)dst_fixed + interpolation_data[i].ss.offset);			
					rss = ((CPixel & 0xff0000) >> 16);
					gss = ((CPixel & 0xff00) >> 8);
					bss = (CPixel & 0xff);
					
					if(ew) {
						CurrentRow = (Pixel *)((char *)dst_fixed + y * dst_pitch);
						radj = ((CurrentRow[x + ew_val] & 0xff0000) >> 16);
						gadj = ((CurrentRow[x + ew_val] & 0xff00) >> 8);
						badj = (CurrentRow[x + ew_val] & 0xff);

						CPixel = Prevlogo[bx+ew_val+(by)*bwidth];
						p_radj = ((CPixel & 0xff0000) >> 16);
						p_gadj = ((CPixel & 0xff00) >> 8); 
						p_badj = (CPixel & 0xff);
					}
					
					if(ns && (!ew)) {
						CurrentRow = (Pixel *)((char *)dst_fixed + (y + ns_val) * dst_pitch);
						radj = ((CurrentRow[x] & 0xff0000) >> 16);
						gadj = ((CurrentRow[x] & 0xff00) >> 8);
						badj = (CurrentRow[x] & 0xff);

						CPixel = Prevlogo[bx+(by+ns_val)*bwidth];
						p_radj = ((CPixel & 0xff0000) >> 16);
						p_gadj = ((CPixel & 0xff00) >> 8); 
						p_badj = (CPixel & 0xff);

					} else if(ns && ew) {
						CurrentRow = (Pixel *)((char *)dst_fixed + (y + ns_val) * dst_pitch);
						radj = (radj+((CurrentRow[x] & 0xff0000) >> 16))/2;
						gadj = (gadj+((CurrentRow[x] & 0xff00) >> 8))/2;
						badj = (badj+(CurrentRow[x] & 0xff))/2;

						CPixel = Prevlogo[bx+(by+ns_val)*bwidth];
						p_radj = (p_radj + ((CPixel & 0xff0000) >> 16))/2;
						p_gadj = (p_gadj + ((CPixel & 0xff00) >> 8))/2; 
						p_badj = (p_badj + (CPixel & 0xff))/2;
					}
					//				ravg+=radj;
					//				gavg+=gadj;
					//				bavg+=badj;
					
					//NS
					rtemp = (rnn*ssd + rss*nnd) / (nnd+ssd);
					gtemp = (gnn*ssd + gss*nnd) / (nnd+ssd);
					btemp = (bnn*ssd + bss*nnd) / (nnd+ssd);
					
					diff_ns += abs(rtemp-radj) + abs(gtemp-gadj) + abs(btemp-badj);
					
					//EW
					rtemp = (ree*wwd + rww*eed) / (eed+wwd);
					gtemp = (gee*wwd + gww*eed) / (eed+wwd);
					btemp = (bee*wwd + bww*eed) / (eed+wwd);
					
					diff_ew += abs(rtemp-radj) + abs(gtemp-gadj) + abs(btemp-badj);
					
					if( abs(p_radj - radj ) > SceneThreshold || 
						abs(p_gadj - gadj ) > SceneThreshold || 
						abs(p_badj - badj ) > SceneThreshold ) 
					{
						SceneChangeTotal++;
					}
												
					PrevFrameBorder[NumPoints_current].r = radj;
					PrevFrameBorder[NumPoints_current].g = gadj;
					PrevFrameBorder[NumPoints_current].b = badj;
					NumPoints_current++;
				}	
			}
			dst	= (Pixel *)((char *)dst + dst_pitch);
			y++;
		}
		
		// find the lowest difference
		
		if(diff_ns <= diff_ew)  weight = 1;
		else weight = 2;
		
		if((SceneChangeTotal*100)/NumPoints > mfd->ScenePercent) SceneChange = true;

	// find avg border rgb
//		if(NumPoints) {
//			ravg/=NumPoints;
//			gavg/=NumPoints;
//			bavg/=NumPoints;
//		}

	}


	if(mfd->debug) { // debug output
		fileptr=fopen("c:/xlogo-debug.txt","w");
		fprintf(fileptr,"Frame: %d\n", frame);
		if(ALPHA<1.0) {
			fprintf(fileptr,"Diff_ns: %5d Diff_ew: %5d\n", diff_ns, diff_ew);
			fprintf(fileptr,"Scene Change: %d\n", (SceneChangeTotal*100)/NumPoints);
		}

		fprintf(fileptr,"Bitmap mask:\n\n");
		for(y=0; y < bheight; y++) {
			for (x = 0; x < bwidth; x++) {
				fprintf(fileptr,"%2d",scalar_table2[x+y*bwidth]);
			}
			fprintf(fileptr,"\n");
		}
		fprintf(fileptr,"\n");
		
	}


	if(ALPHA<1.0 && SceneChange  && (mfd->ModeInterpolate==MI_NORMAL || mfd->ModeInterpolate==MI_USE_1)) {
		mfd->PrevWeight = weight;

		y = height - Y +1 - bheight; // absolute frame y coord
		dst	= (Pixel *)((char *)dst_fixed + y * dst_pitch);
		for (by = 1; by < bheight-1; by++) {
			for (x = X+1, bx = 1; x < X + bwidth-1; x++, bx++)	{
				r = ((dst[x] & 0xff0000) >> 16);
				g = ((dst[x] & 0xff00) >> 8);
				b = (dst[x] & 0xff);
				
				i = bx+by*bwidth;
				ns = ns_table[i]!=0;
				ew = ew_table[i]!=0;
				
				if(scalar_table[i]==0) {
					
					nnd = interpolation_data[i].nn.d;
					eed = interpolation_data[i].ee.d;
					ssd = interpolation_data[i].ss.d;
					wwd = interpolation_data[i].ww.d;
					
					CPixel = *(Pixel *)((char *)dst_fixed + interpolation_data[i].ee.offset);			
					ree = ((CPixel & 0xff0000) >> 16);
					gee = ((CPixel & 0xff00) >> 8);
					bee = (CPixel & 0xff);

					CPixel = *(Pixel *)((char *)dst_fixed + interpolation_data[i].ww.offset);			
					rww = ((CPixel & 0xff0000) >> 16);
					gww = ((CPixel & 0xff00) >> 8);
					bww = (CPixel & 0xff);

					CPixel = *(Pixel *)((char *)dst_fixed + interpolation_data[i].nn.offset);			
					rnn = ((CPixel & 0xff0000) >> 16);
					gnn = ((CPixel & 0xff00) >> 8);
					bnn = (CPixel & 0xff);

					CPixel = *(Pixel *)((char *)dst_fixed + interpolation_data[i].ss.offset);			
					rss = ((CPixel & 0xff0000) >> 16);
					gss = ((CPixel & 0xff00) >> 8);
					bss = (CPixel & 0xff);

					if(weight==1) {
						//NS
						if(abs(rnn-rss) < same_threshold && abs(gnn-gss) < same_threshold && abs(bnn-bss) < same_threshold) {
							r = (rnn*ssd + rss*nnd) / (nnd+ssd);
							g = (gnn*ssd + gss*nnd) / (nnd+ssd);
							b = (bnn*ssd + bss*nnd) / (nnd+ssd);
							scalar_table2[i] = 2;
						}
					} else {
						//EW
						if(abs(ree-rww) < same_threshold && abs(gee-gww) < same_threshold && abs(bee-bww) < same_threshold) {
							r = (ree*wwd + rww*eed) / (eed+wwd);
							g = (gee*wwd + gww*eed) / (eed+wwd);
							b = (bee*wwd + bww*eed) / (eed+wwd);
							scalar_table2[i] = 2;
						}
					}
					dst[x] = ((r << 16) | (g << 8) | (b));
				}	
			}
			dst	= (Pixel *)((char *)dst + dst_pitch);
			y++;
		}

		/////////////
		if(mfd->debug) { // debug output
			fprintf(fileptr,"First pass:  Weight = %d\n\n",weight);
			for(y=0; y < bheight; y++) {
				for (x = 0; x < bwidth; x++) {
					fprintf(fileptr,"%2d",scalar_table2[x+y*bwidth]);
				}
				fprintf(fileptr,"\n");
			}
			fprintf(fileptr,"\n");
		}
		/////////////

		// make another pass with the opposite weight;
		if(weight==1) weight=2;
		else weight=1;


		y = height - Y +1 - bheight; // absolute frame y coord
		dst	= (Pixel *)((char *)dst_fixed + y * dst_pitch);
		for (by = 1; by < bheight-1; by++) {
			for (x = X+1, bx = 1; x < X + bwidth-1; x++, bx++)	{

				i = bx+by*bwidth;
				if(scalar_table2[i]==0) {
					r = ((dst[x] & 0xff0000) >> 16);
					g = ((dst[x] & 0xff00) >> 8);
					b = (dst[x] & 0xff);

					if(weight==1) {
						//NS

						//get north pt
						dy = 0;
						cval=0;
						while(cval==0) {
							dy--;
							cval = scalar_table2[bx+(by+dy)*bwidth];
						}
						nnd = abs(dy);
						offset = x*4 + (y + dy) * dst_pitch;
						CPixel = *(Pixel *)((char *)dst_fixed + offset);			
						rnn = ((CPixel & 0xff0000) >> 16);
						gnn = ((CPixel & 0xff00) >> 8);
						bnn = (CPixel & 0xff);

						//south
						dy = 0;
						cval=0;
						while(cval==0) {
							dy++;
							cval = scalar_table2[bx+(by+dy)*bwidth];
						}
						ssd = dy;
						offset = x*4 + (y + dy) * dst_pitch;
						CPixel = *(Pixel *)((char *)dst_fixed + offset);			
						rss = ((CPixel & 0xff0000) >> 16);
						gss = ((CPixel & 0xff00) >> 8);
						bss = (CPixel & 0xff);

						if(abs(rnn-rss) < same_threshold2 && abs(gnn-gss) < same_threshold2 && abs(bnn-bss) < same_threshold2) {
							r = (rnn*ssd + rss*nnd) / (nnd+ssd);
							g = (gnn*ssd + gss*nnd) / (nnd+ssd);
							b = (bnn*ssd + bss*nnd) / (nnd+ssd);

							scalar_table2[i] = 3;
						} 

					} else  {
						//EW

						//east
						dx = 0;
						cval=0;
						while(cval==0) {
							dx++;
							cval = scalar_table2[bx+dx+by*bwidth];
						}
						eed = dx;
						offset = (x + dx)*4 + (y) * dst_pitch;
						CPixel = *(Pixel *)((char *)dst_fixed + offset);			
						ree = ((CPixel & 0xff0000) >> 16);
						gee = ((CPixel & 0xff00) >> 8);
						bee = (CPixel & 0xff);

						//west
						dx = 0;
						cval=0;
						while(cval==0) {
							dx--;
							cval = scalar_table2[bx+dx+by*bwidth];
						}
						wwd = abs(dx);
						offset = (x + dx)*4 + (y) * dst_pitch;
						CPixel = *(Pixel *)((char *)dst_fixed + offset);			
						rww = ((CPixel & 0xff0000) >> 16);
						gww = ((CPixel & 0xff00) >> 8);
						bww = (CPixel & 0xff);

						if(abs(ree-rww) < same_threshold2 && abs(gee-gww) < same_threshold2 && abs(bee-bww) < same_threshold2) {
							r = (ree*wwd + rww*eed) / (eed+wwd);
							g = (gee*wwd + gww*eed) / (eed+wwd);
							b = (bee*wwd + bww*eed) / (eed+wwd);

							scalar_table2[i] = 3;
						} 

					}

					dst[x] = ((r << 16) | (g << 8) | (b));
				}	
			}
			dst	= (Pixel *)((char *)dst + dst_pitch);
			y++;
		} 

		//////////////
		if(mfd->debug) { // debug output
			fprintf(fileptr,"2nd pass:  Weight = %d\n\n",weight);
			for(y=0; y < bheight; y++) {
				for (x = 0; x < bwidth; x++) {
					fprintf(fileptr,"%2d",scalar_table2[x+y*bwidth]);
				}
				fprintf(fileptr,"\n");
			}
			fprintf(fileptr,"\n");
		}
		//////////////

		int passT3;
		int diffr, diffg, diffb;

		// anything left do with equal weighting
		y = height - Y +1 - bheight; // absolute frame y coord
		dst	= (Pixel *)((char *)dst_fixed + y * dst_pitch);
		for (by = 1; by < bheight-1; by++) {
			for (x = X+1, bx = 1; x < X + bwidth-1; x++, bx++)	{

				i = bx+by*bwidth;
				if(scalar_table2[i]==0) {
					r = ((dst[x] & 0xff0000) >> 16);
					g = ((dst[x] & 0xff00) >> 8);
					b = (dst[x] & 0xff);
					
					//get north pt
					dy = 0;
					cval=0;
					while(cval==0) {
						dy--;
						cval = scalar_table2[bx+(by+dy)*bwidth];
					}
					nnd = abs(dy);
					offset = x*4 + (y + dy) * dst_pitch;
					CPixel = *(Pixel *)((char *)dst_fixed + offset);			
					rnn = ((CPixel & 0xff0000) >> 16);
					gnn = ((CPixel & 0xff00) >> 8);
					bnn = (CPixel & 0xff);
					
					//south
					dy = 0;
					cval=0;
					while(cval==0) {
						dy++;
						cval = scalar_table2[bx+(by+dy)*bwidth];
					}
					ssd = dy;
					offset = x*4 + (y + dy) * dst_pitch;
					CPixel = *(Pixel *)((char *)dst_fixed + offset);			
					rss = ((CPixel & 0xff0000) >> 16);
					gss = ((CPixel & 0xff00) >> 8);
					bss = (CPixel & 0xff);
					
					//east
					dx = 0;
					cval=0;
					while(cval==0) {
						dx++;
						cval = scalar_table2[bx+dx+by*bwidth];
					}
					eed = dx;
					offset = (x + dx)*4 + (y) * dst_pitch;
					CPixel = *(Pixel *)((char *)dst_fixed + offset);			
					ree = ((CPixel & 0xff0000) >> 16);
					gee = ((CPixel & 0xff00) >> 8);
					bee = (CPixel & 0xff);
					
					//west
					dx = 0;
					cval=0;
					while(cval==0) {
						dx--;
						cval = scalar_table2[bx+dx+by*bwidth];
					}
					wwd = abs(dx);
					offset = (x + dx)*4 + (y) * dst_pitch;
					CPixel = *(Pixel *)((char *)dst_fixed + offset);			
					rww = ((CPixel & 0xff0000) >> 16);
					gww = ((CPixel & 0xff00) >> 8);
					bww = (CPixel & 0xff);

					passT3 = 0;
					
					diffr = abs(rnn-rss) - abs(rww-ree);
					diffg = abs(gnn-gss) - abs(gww-gee);
					diffb = abs(bnn-bss) - abs(bww-bee);

					if(diffr > same_threshold3 && diffg > same_threshold3 && diffb > same_threshold3) passT3 = 2;
					if(diffr < -same_threshold3 && diffg < -same_threshold3 && diffb < -same_threshold3) passT3 = 1;

					if(scalar_table2[i]==0) {
						if(passT3==0) {						
							r = (rnn*ssd + ree*wwd + rss*nnd + rww*eed) / (nnd+ssd+eed+wwd);
							g = (gnn*ssd + gee*wwd + gss*nnd + gww*eed) / (nnd+ssd+eed+wwd);
							b = (bnn*ssd + bee*wwd + bss*nnd + bww*eed) / (nnd+ssd+eed+wwd);
						} else if(passT3==1) {
							r = (rnn*ssd + rss*nnd) / (nnd+ssd);
							g = (gnn*ssd + gss*nnd) / (nnd+ssd);
							b = (bnn*ssd + bss*nnd) / (nnd+ssd);
						} else {
							r = (ree*wwd + rww*eed) / (eed+wwd);
							g = (gee*wwd + gww*eed) / (eed+wwd);
							b = (bee*wwd + bww*eed) / (eed+wwd);
						}	

					} 

					dst[x] = ((r << 16) | (g << 8) | (b));
				}	
			}
			dst	= (Pixel *)((char *)dst + dst_pitch);
			y++;
		}
		//////////////
		if(mfd->debug) { // debug output
			fprintf(fileptr,"2nd pass:  Weight = %d\n\n",weight);
			for(y=0; y < bheight; y++) {
				for (x = 0; x < bwidth; x++) {
					fprintf(fileptr,"%2d",scalar_table2[x+y*bwidth]);
				}
				fprintf(fileptr,"\n");
			}
			fprintf(fileptr,"\n");
		}
		//////////////

		//  output to prevlogo buffer;
		y = height - Y - bheight; // absolute frame y coord
		i = 0;
		dst	= (Pixel *)((char *)dst_fixed + y * dst_pitch);
		for (by = 0; by < bheight; by++) {
			for (x = X, bx = 0; x < X + bwidth; x++, bx++)	{
				Prevlogo[i] = dst[x];
				i++;
			}
			dst	= (Pixel *)((char *)dst + dst_pitch);
			y++;
		}
	}
	if(mfd->ModeInterpolate==MI_EAT) {


	}

	if(ALPHA<1.0) {
		if(!SceneChange) {
			// copy prevlogo buffer to dst
			y = height - Y - bheight; // absolute frame y coord
			i = 0;
			dst	= (Pixel *)((char *)dst_fixed + y * dst_pitch);
			for (by = 0; by < bheight; by++) {
				for (x = X, bx = 0; x < X + bwidth; x++, bx++)	{
					j = bx+by*bwidth;
				
					if(scalar_table[j]==0) {
						dst[x] = Prevlogo[i];
					}
					i++;
				}
				dst	= (Pixel *)((char *)dst + dst_pitch);
				y++;
			}
	
		}
		//blur logo area

		// at this point scalar_table2[i]  1=non logo portion  2=1st pass  3=2nd pass  0=3rd pass
		// reset it to contain the number of blur passes

		for(i=0; i < bheight*bwidth; i++) {
			switch(scalar_table2[i]) {
			case 1:
				scalar_table2[i] = 0;
				break;
			case 2:
				scalar_table2[i] = blur1;
				break;
			case 3:
				scalar_table2[i] = blur2;
				break;
			case 0:
				scalar_table2[i] = blur3;
				break;
			}
		}

		unsigned int passes = blur3;
		if(blur2 > passes) passes = blur2;
		if(blur1 > passes) passes = blur1;

		if(passes) {
			BlurArea(mfd, fa, logobuffer1, logobuffer2, scalar_table2, passes);
		}
	}

	//////////////
	if(mfd->debug) { // debug output
		fclose(fileptr);
	}
	//////////////



	if(mfd->ModeTransparent==MT_NONE) {
		// alpha blend src
		if(ALPHA>0.0 && ALPHA < 1.0) {
			y = height - Y - bheight; // absolute frame y coord
			dst	= (Pixel *)((char *)dst_fixed + y * dst_pitch);
			src	= (Pixel *)((char *)src_fixed + y * src_pitch);
			for (by = 0; by < bheight; by++) {
				for (x = X, bx = 0; x < X + bwidth; x++, bx++)	{
					int i = bx+by*bwidth;
					if(mfd->scalar_table[i]==0) {
						r1 = ((src[x] & 0xff0000) >> 16);
						g1 = ((src[x] & 0xff00) >> 8);
						b1 = (src[x] & 0xff);
						r2 = ((dst[x] & 0xff0000) >> 16);
						g2 = ((dst[x] & 0xff00) >> 8);
						b2 = (dst[x] & 0xff);
					
						r = (int) (ALPHA * r1 + (1 - ALPHA) * r2);
						g = (int) (ALPHA * g1 + (1 - ALPHA) * g2);
						b = (int) (ALPHA * b1 + (1 - ALPHA) * b2);
					
						dst[x] = ((r << 16) | (g << 8) | (b));
					}
				}
				dst	= (Pixel *)((char *)dst + dst_pitch);
				src	= (Pixel *)((char *)src + src_pitch);
				y++;
			}
		}
	} else {
		// alpha blend transparent output (in buffer 3)
		if(ALPHA>0.0 && ALPHA < 1.0) {
			y = height - Y - bheight; // absolute frame y coord
			dst	= (Pixel *)((char *)dst_fixed + y * dst_pitch);
			for (by = 0; by < bheight; by++) {
				for (x = X, bx = 0; x < X + bwidth; x++, bx++)	{
					int i = bx+by*bwidth;
					if(mfd->scalar_table[i]==0) {
						r1 = logobuffer3[i].r;
						g1 = logobuffer3[i].g;
						b1 = logobuffer3[i].b;
						r2 = ((dst[x] & 0xff0000) >> 16);
						g2 = ((dst[x] & 0xff00) >> 8);
						b2 = (dst[x] & 0xff);
					
						r = (int) (ALPHA * r1 + (1 - ALPHA) * r2);
						g = (int) (ALPHA * g1 + (1 - ALPHA) * g2);
						b = (int) (ALPHA * b1 + (1 - ALPHA) * b2);
					
						dst[x] = ((r << 16) | (g << 8) | (b));
					}
				}
				dst	= (Pixel *)((char *)dst + dst_pitch);
				y++;
			}
		} else if(ALPHA==1.0) {
			y = height - Y - bheight; // absolute frame y coord
			dst	= (Pixel *)((char *)dst_fixed + y * dst_pitch);
			for (by = 0; by < bheight; by++) {
				for (x = X, bx = 0; x < X + bwidth; x++, bx++)	{
					int i = bx+by*bwidth;
					if(mfd->scalar_table[i]==0) {
						r1 = logobuffer3[i].r;
						g1 = logobuffer3[i].g;
						b1 = logobuffer3[i].b;
					
						dst[x] = ((r1 << 16) | (g1 << 8) | (b1));
					}
				}
				dst	= (Pixel *)((char *)dst + dst_pitch);
				y++;
			}
		}
	}	
	//blur bitmap area

	if(blurbitmap) {
		BlurArea(mfd, fa, logobuffer1, logobuffer2, mfd->scalar_table3, blurbitmap);
	}

	if(mfd->noise_level) {
		int noise_level = mfd->noise_level;
		int noise_level_X2 = noise_level*2;
		int rtemp, gtemp, btemp;
		y = height - Y - bheight; // absolute frame y coord
		dst	= (Pixel *)((char *)dst_fixed + y * dst_pitch);
		for (by = 0; by < bheight; by++) {
			for (x = X, bx = 0; x < X + bwidth; x++, bx++)	{
				rtemp = ((dst[x] & 0xff0000) >> 16) + rand() % noise_level_X2 - noise_level;
				gtemp = ((dst[x] & 0xff00) >> 8) + rand() % noise_level_X2 - noise_level;
				btemp = (dst[x] & 0xff) + rand() % noise_level_X2 - noise_level;
				if(rtemp<0)		rtemp=0;
				if(rtemp>255)	rtemp=255;
				if(gtemp<0)		gtemp=0;
				if(gtemp>255)	gtemp=255;
				if(btemp<0)		btemp=0;
				if(btemp>255)	btemp=255;

				dst[x] = ((rtemp << 16) | (gtemp << 8) | (btemp));
			}
			dst	= (Pixel *)((char *)dst + dst_pitch);
			y++;
		}
	}
	return 0;
}

extern "C" int __declspec(dllexport) __cdecl VirtualdubFilterModuleInit2(FilterModule *fm, const FilterFunctions *ff, int& vdfd_ver, int& vdfd_compat);
extern "C" void __declspec(dllexport) __cdecl VirtualdubFilterModuleDeinit(FilterModule *fm, const FilterFunctions *ff);

static FilterDefinition *fd_tutorial;

int __declspec(dllexport) __cdecl VirtualdubFilterModuleInit2(FilterModule *fm, const FilterFunctions *ff, int& vdfd_ver, int& vdfd_compat) {
	if (!(fd_tutorial = ff->addFilter(fm, &filterDef_tutorial, sizeof(FilterDefinition))))
		return 1;

	vdfd_ver = VIRTUALDUB_FILTERDEF_VERSION;
	vdfd_compat = VIRTUALDUB_FILTERDEF_COMPATIBLE;

	return 0;
}

void __declspec(dllexport) __cdecl VirtualdubFilterModuleDeinit(FilterModule *fm, const FilterFunctions *ff) {
	ff->removeFilter(fd_tutorial);
}



BOOL CALLBACK ConfigDlgProc(HWND hdlg, UINT msg, WPARAM wParam, LPARAM lParam) {
	MyFilterData *mfd = (MyFilterData *)GetWindowLong(hdlg, DWL_USER);

	switch(msg) {
		case WM_INITDIALOG:
			SetWindowLong(hdlg, DWL_USER, lParam);
			mfd = (MyFilterData *)lParam;

			CheckDlgButton(hdlg, IDC_DEBUG, mfd->debug ? BST_CHECKED : BST_UNCHECKED);
			SetWindowText(GetDlgItem(hdlg, IDC_FILENAME), mfd->filename);

			SendMessage(GetDlgItem(hdlg, IDC_XSLIDER), TBM_SETRANGE, (WPARAM)TRUE, MAKELONG(0, 800));
			SendMessage(GetDlgItem(hdlg, IDC_XSLIDER), TBM_SETPOS, (WPARAM)TRUE, mfd->x);
			SetDlgItemInt(hdlg, IDC_XOFF, mfd->x, FALSE);

			SendMessage(GetDlgItem(hdlg, IDC_YSLIDER), TBM_SETRANGE, (WPARAM)TRUE, MAKELONG(0, 600));
			SendMessage(GetDlgItem(hdlg, IDC_YSLIDER), TBM_SETPOS, (WPARAM)TRUE, mfd->y);
			SetDlgItemInt(hdlg, IDC_YOFF, mfd->y, FALSE);
			
			SendMessage(GetDlgItem(hdlg, IDC_SALPHA), TBM_SETRANGE, (WPARAM)TRUE, MAKELONG(0, 255));
			SendMessage(GetDlgItem(hdlg, IDC_SALPHA), TBM_SETPOS, (WPARAM)TRUE, mfd->alpha);
			SetDlgItemInt(hdlg, IDC_ALPHADISPLAY, mfd->alpha, FALSE);

			SetDlgItemInt(hdlg, IDC_FADEOUT, mfd->fadeout, FALSE);
			SetDlgItemInt(hdlg, IDC_START, mfd->start, FALSE);
			SetDlgItemInt(hdlg, IDC_DURATION, mfd->duration, FALSE);
			SetDlgItemInt(hdlg, IDC_FADEIN, mfd->fadein, FALSE);
			SetDlgItemInt(hdlg, IDC_SCENE, mfd->ScenePercent, FALSE);
			SetDlgItemInt(hdlg, IDC_THRESHOLD1, mfd->threshold1, FALSE);
			SetDlgItemInt(hdlg, IDC_THRESHOLD2, mfd->threshold2, FALSE);
			SetDlgItemInt(hdlg, IDC_THRESHOLD3, mfd->threshold3, FALSE);
			SetDlgItemInt(hdlg, IDC_BLUR2, mfd->blur2, FALSE);
			SetDlgItemInt(hdlg, IDC_BLUR3, mfd->blur3, FALSE);
			SetDlgItemInt(hdlg, IDC_BLURBITMAP, mfd->blurbitmap, FALSE);
			SetDlgItemInt(hdlg, IDC_WHITE, mfd->white, FALSE);
			SetDlgItemInt(hdlg, IDC_FEATHER, mfd->feather, FALSE);
			SetDlgItemInt(hdlg, IDC_NOISE_LVL, mfd->noise_level, FALSE);

			if(mfd->interpolate_side==0) CheckRadioButton(hdlg, IDC_RADIO1, IDC_RADIO5, IDC_RADIO5);
			else if(mfd->interpolate_side==1) CheckRadioButton(hdlg, IDC_RADIO1, IDC_RADIO5, IDC_RADIO1);
			else if(mfd->interpolate_side==2) CheckRadioButton(hdlg, IDC_RADIO1, IDC_RADIO5, IDC_RADIO2);
			else if(mfd->interpolate_side==3) CheckRadioButton(hdlg, IDC_RADIO1, IDC_RADIO5, IDC_RADIO3);
			else if(mfd->interpolate_side==4) CheckRadioButton(hdlg, IDC_RADIO1, IDC_RADIO5, IDC_RADIO4);

			char back1[50], back2[50], back3[50];
			if(mfd->ModeTransparent==MT_NONE) {
				sprintf(back1,"N/A");
				sprintf(back2,"N/A");
				SendDlgItemMessage(hdlg, IDC_WHITE, WM_ENABLE,(WPARAM)false,NULL);
			} else if(mfd->ModeTransparent==MT_WHITE) {
				sprintf(back1,"(%d,%d,%d)",mfd->back1r, mfd->back1g,mfd->back1b);
				sprintf(back2,"Enter \\/");
				SendDlgItemMessage(hdlg, IDC_WHITE, WM_ENABLE,(WPARAM)true,NULL);
			} else {
				sprintf(back1,"(%d,%d,%d)",mfd->back1r, mfd->back1g,mfd->back1b);
				sprintf(back2,"(%d,%d,%d)",mfd->back2r, mfd->back2g,mfd->back2b);
				SendDlgItemMessage(hdlg, IDC_WHITE, WM_ENABLE,(WPARAM)false,NULL);
			}
			if(mfd->ModeInterpolate==MI_NONE) {
				sprintf(back3,"N/A");
			} else if(mfd->ModeInterpolate==MI_NORMAL) {
				sprintf(back3,"(%d,%d,%d)",mfd->back3r, mfd->back3g,mfd->back3b);
			} else if(mfd->ModeInterpolate==MI_USE_1) {
				sprintf(back3,"<- Using Black");
			} else if(mfd->ModeInterpolate==MI_EAT) {
				sprintf(back3,"Experimental");
			}
			SetWindowText(GetDlgItem(hdlg, IDC_BACK1), back1);
			SetWindowText(GetDlgItem(hdlg, IDC_BACK2), back2);
			SetWindowText(GetDlgItem(hdlg, IDC_BACK3), back3);

			if(mfd->alpha==255) {
				//disable interpolation-only controls
				SendDlgItemMessage(hdlg, IDC_THRESHOLD1, WM_ENABLE,(WPARAM)false,NULL);
				SendDlgItemMessage(hdlg, IDC_THRESHOLD2, WM_ENABLE,(WPARAM)false,NULL);
				SendDlgItemMessage(hdlg, IDC_THRESHOLD3, WM_ENABLE,(WPARAM)false,NULL);
				SendDlgItemMessage(hdlg, IDC_BLUR2, WM_ENABLE,(WPARAM)false,NULL);
				SendDlgItemMessage(hdlg, IDC_BLUR3, WM_ENABLE,(WPARAM)false,NULL);
				SendDlgItemMessage(hdlg, IDC_SCENE, WM_ENABLE,(WPARAM)false,NULL);
			}
			mfd->ifp->InitButton(GetDlgItem(hdlg, IDPREVIEW));
			return TRUE;

		case WM_HSCROLL:
			if ((HWND) lParam == GetDlgItem(hdlg, IDC_XSLIDER))
			{
				int x = SendMessage(GetDlgItem(hdlg, IDC_XSLIDER), TBM_GETPOS, 0, 0);
				if (x != mfd->x)
				{
					mfd->x = x;
					SetDlgItemInt(hdlg, IDC_XOFF, mfd->x, FALSE);
					mfd->ifp->RedoFrame();
				}
			}
			if ((HWND) lParam == GetDlgItem(hdlg, IDC_SALPHA))
			{
				int alpha = SendMessage(GetDlgItem(hdlg, IDC_SALPHA), TBM_GETPOS, 0, 0);
				if (alpha != mfd->alpha)
				{
					if(mfd->alpha==255) {
						//enable interpolation-only controls
						SendDlgItemMessage(hdlg, IDC_THRESHOLD1, WM_ENABLE,(WPARAM)true,NULL);
						SendDlgItemMessage(hdlg, IDC_THRESHOLD2, WM_ENABLE,(WPARAM)true,NULL);
						SendDlgItemMessage(hdlg, IDC_THRESHOLD3, WM_ENABLE,(WPARAM)true,NULL);
						SendDlgItemMessage(hdlg, IDC_BLUR2, WM_ENABLE,(WPARAM)true,NULL);
						SendDlgItemMessage(hdlg, IDC_BLUR3, WM_ENABLE,(WPARAM)true,NULL);
						SendDlgItemMessage(hdlg, IDC_SCENE, WM_ENABLE,(WPARAM)true,NULL);
					}
					mfd->alpha = alpha;
					SetDlgItemInt(hdlg, IDC_ALPHADISPLAY, mfd->alpha, FALSE);
					if(alpha==255) {
						//disable interpolation-only controls
						SendDlgItemMessage(hdlg, IDC_THRESHOLD1, WM_ENABLE,(WPARAM)false,NULL);
						SendDlgItemMessage(hdlg, IDC_THRESHOLD2, WM_ENABLE,(WPARAM)false,NULL);
						SendDlgItemMessage(hdlg, IDC_THRESHOLD3, WM_ENABLE,(WPARAM)false,NULL);
						SendDlgItemMessage(hdlg, IDC_BLUR2, WM_ENABLE,(WPARAM)false,NULL);
						SendDlgItemMessage(hdlg, IDC_BLUR3, WM_ENABLE,(WPARAM)false,NULL);
						SendDlgItemMessage(hdlg, IDC_SCENE, WM_ENABLE,(WPARAM)false,NULL);
					}
					mfd->ifp->RedoFrame();
				}
			}
			break;
		case WM_VSCROLL:
			if ((HWND) lParam == GetDlgItem(hdlg, IDC_YSLIDER))
			{
				int y = SendMessage(GetDlgItem(hdlg, IDC_YSLIDER), TBM_GETPOS, 0, 0);
				if (y != mfd->y)
				{
					mfd->y = y;
					SetDlgItemInt(hdlg, IDC_YOFF, mfd->y, FALSE);
					mfd->ifp->RedoFrame();
				}
			}
			break;

		case WM_COMMAND:
			switch(LOWORD(wParam)) {

			case IDPREVIEW:
				mfd->ifp->Toggle(hdlg);
				break;

			case IDOK:
				EndDialog(hdlg, 0);
				return TRUE;
			case IDHELP:
				{
				char prog[256];
				char path[256];
				LPTSTR ptr;

				GetModuleFileName(NULL, prog, 255);
				GetFullPathName(prog, 255, path, &ptr);
				*ptr = 0;
				strcat(path, "plugins\\xlogo\\Xlogo.html");
				ShellExecute(hdlg, "open", path, NULL, NULL, SW_SHOWNORMAL);
				return TRUE;
				}

			case IDC_BROWSE:
				{
				OPENFILENAME ofn;
				mfd->filename[0] = NULL;
				ofn.lStructSize = sizeof(OPENFILENAME);
				ofn.hwndOwner = hdlg;
				ofn.hInstance = NULL;
				ofn.lpTemplateName = NULL;
				ofn.lpstrFilter = "Bitmaps\0*.BMP\0All Files\0*.*\0\0";
				ofn.lpstrCustomFilter = NULL;
				ofn.nMaxCustFilter = 0;
				ofn.nFilterIndex = 1;
				ofn.lpstrFile = mfd->filename;
				ofn.nMaxFile = 1024;
				ofn.lpstrFileTitle = NULL;
				ofn.nMaxFileTitle = 0;
				ofn.lpstrInitialDir = NULL;
				ofn.lpstrTitle = "Bitmap Filename must end in 0 or 2";
				ofn.Flags = OFN_CREATEPROMPT;
				ofn.nFileOffset = 0;
				ofn.nFileExtension = 0;
				ofn.lpstrDefExt = NULL;
				ofn.lCustData = 0;
				ofn.lpfnHook = NULL;
				GetOpenFileName(&ofn);
				if (mfd->filename[0] != 0)
				{
					SetWindowText(GetDlgItem(hdlg, IDC_FILENAME), mfd->filename);
					LoadBitmaps(mfd);
					char* teststring;
					teststring = strstr(mfd->filename,"_x_");
					if(teststring!=NULL) {
						mfd->x = atoi(teststring+3);
						if(mfd->x < 0) mfd->x = 0;
						if(mfd->x > 800) mfd->x = 800;
						SendMessage(GetDlgItem(hdlg, IDC_XSLIDER), TBM_SETPOS, (WPARAM)TRUE, mfd->x);
						SetDlgItemInt(hdlg, IDC_XOFF, mfd->x, FALSE);
					}
					teststring = strstr(mfd->filename,"_y_");
					if(teststring!=NULL) {
						mfd->y = atoi(teststring+3);
						if(mfd->y < 0) mfd->y = 0;
						if(mfd->y > 600) mfd->y = 600;
						SendMessage(GetDlgItem(hdlg, IDC_YSLIDER), TBM_SETPOS, (WPARAM)TRUE, mfd->y);
						SetDlgItemInt(hdlg, IDC_YOFF, mfd->y, FALSE);
					}

					char back1[50], back2[50], back3[50];
					if(mfd->ModeTransparent==MT_NONE) {
						sprintf(back1,"N/A");
						sprintf(back2,"N/A");
						SendDlgItemMessage(hdlg, IDC_WHITE, WM_ENABLE,(WPARAM)false,NULL);
						if(mfd->alpha!=0) {
							mfd->alpha = 0;
							SendMessage(GetDlgItem(hdlg, IDC_SALPHA), TBM_SETPOS, (WPARAM)TRUE, mfd->alpha);
							SetDlgItemInt(hdlg, IDC_ALPHADISPLAY, mfd->alpha, FALSE);
						}
					} else if(mfd->ModeTransparent==MT_WHITE) {
						sprintf(back1,"(%d,%d,%d)",mfd->back1r, mfd->back1g,mfd->back1b);
						sprintf(back2,"Enter \\/");
						SendDlgItemMessage(hdlg, IDC_WHITE, WM_ENABLE,(WPARAM)true,NULL);
					} else {
						sprintf(back1,"(%d,%d,%d)",mfd->back1r, mfd->back1g,mfd->back1b);
						sprintf(back2,"(%d,%d,%d)",mfd->back2r, mfd->back2g,mfd->back2b);
						SendDlgItemMessage(hdlg, IDC_WHITE, WM_ENABLE,(WPARAM)false,NULL);

					}
					if(mfd->ModeTransparent==MT_WHITE || MT_COLOR) {
						mfd->alpha = 255+DEFAULT_ALPHA_T-mfd->bitmap_difficulty;
						if(mfd->alpha>255) mfd->alpha = 255;
						if(mfd->alpha<DEFAULT_ALPHA_MIN) mfd->alpha = 150;
						if(SendMessage(GetDlgItem(hdlg, IDC_SALPHA), TBM_GETPOS, 0, 0) != mfd->alpha) {
							SendMessage(GetDlgItem(hdlg, IDC_SALPHA), TBM_SETPOS, (WPARAM)TRUE, mfd->alpha);
							SetDlgItemInt(hdlg, IDC_ALPHADISPLAY, mfd->alpha, FALSE);
							if(mfd->alpha==255) {
								//disable interpolation-only controls
								SendDlgItemMessage(hdlg, IDC_THRESHOLD1, WM_ENABLE,(WPARAM)false,NULL);
								SendDlgItemMessage(hdlg, IDC_THRESHOLD2, WM_ENABLE,(WPARAM)false,NULL);
								SendDlgItemMessage(hdlg, IDC_THRESHOLD3, WM_ENABLE,(WPARAM)false,NULL);
								SendDlgItemMessage(hdlg, IDC_BLUR2, WM_ENABLE,(WPARAM)false,NULL);
								SendDlgItemMessage(hdlg, IDC_BLUR3, WM_ENABLE,(WPARAM)false,NULL);
								SendDlgItemMessage(hdlg, IDC_SCENE, WM_ENABLE,(WPARAM)false,NULL);
							}

						}
					}
					if(mfd->ModeInterpolate==MI_NONE) {
						sprintf(back3,"N/A");
					} else if(mfd->ModeInterpolate==MI_NORMAL || mfd->ModeInterpolate==MI_EAT) {
						if(mfd->ModeInterpolate==MI_NORMAL) {
							sprintf(back3,"(%d,%d,%d)",mfd->back3r, mfd->back3g,mfd->back3b);
						} else if(mfd->ModeInterpolate==MI_EAT) {
							sprintf(back3,"Experimental");
						}
						if(mfd->alpha!=0 && mfd->ModeTransparent==MT_NONE) {
							mfd->alpha=0;
							SendMessage(GetDlgItem(hdlg, IDC_SALPHA), TBM_SETPOS, (WPARAM)TRUE, mfd->alpha);
							SetDlgItemInt(hdlg, IDC_ALPHADISPLAY, mfd->alpha, FALSE);
							//enable interpolation-only controls
							SendDlgItemMessage(hdlg, IDC_THRESHOLD1, WM_ENABLE,(WPARAM)true,NULL);
							SendDlgItemMessage(hdlg, IDC_THRESHOLD2, WM_ENABLE,(WPARAM)true,NULL);
							SendDlgItemMessage(hdlg, IDC_THRESHOLD3, WM_ENABLE,(WPARAM)true,NULL);
							SendDlgItemMessage(hdlg, IDC_BLUR2, WM_ENABLE,(WPARAM)true,NULL);
							SendDlgItemMessage(hdlg, IDC_BLUR3, WM_ENABLE,(WPARAM)true,NULL);
							SendDlgItemMessage(hdlg, IDC_SCENE, WM_ENABLE,(WPARAM)true,NULL);
						}
					} else {
						sprintf(back3,"<- Using Black");
					}
					SetWindowText(GetDlgItem(hdlg, IDC_BACK1), back1);
					SetWindowText(GetDlgItem(hdlg, IDC_BACK2), back2);
					SetWindowText(GetDlgItem(hdlg, IDC_BACK3), back3);

					mfd->ifp->UndoSystem();
					mfd->ifp->RedoSystem();
				}
				break;
				}

			case IDC_DEBUG:
				mfd->debug = !!IsDlgButtonChecked(hdlg, IDC_ANIMATE);
				mfd->ifp->RedoFrame();
				break;
			case IDC_FADEOUT:
				mfd->fadeout = GetDlgItemInt(hdlg, IDC_FADEOUT, 0, FALSE);
				mfd->ifp->RedoFrame();
				break;
			case IDC_START:
				mfd->start = GetDlgItemInt(hdlg, IDC_START, 0, FALSE);
				mfd->ifp->RedoFrame();
				break;
			case IDC_SCENE:
				mfd->ScenePercent = GetDlgItemInt(hdlg, IDC_SCENE, 0, FALSE);
				mfd->ifp->RedoFrame();
				break;
			case IDC_DURATION:
				mfd->duration = GetDlgItemInt(hdlg, IDC_DURATION, 0, FALSE);
				mfd->ifp->RedoFrame();
				break;
			case IDC_THRESHOLD1:
				mfd->threshold1 = GetDlgItemInt(hdlg, IDC_THRESHOLD1, 0, FALSE);
				mfd->ifp->RedoFrame();
				break;
			case IDC_THRESHOLD2:
				mfd->threshold2 = GetDlgItemInt(hdlg, IDC_THRESHOLD2, 0, FALSE);
				mfd->ifp->RedoFrame();
				break;
			case IDC_THRESHOLD3:
				mfd->threshold3 = GetDlgItemInt(hdlg, IDC_THRESHOLD3, 0, FALSE);
				mfd->ifp->RedoFrame();
				break;
			case IDC_BLUR2:
				mfd->blur1 = mfd->blur2 = GetDlgItemInt(hdlg, IDC_BLUR2, 0, FALSE);
				mfd->ifp->RedoFrame();
				break;
			case IDC_BLUR3:
				mfd->blur3 = GetDlgItemInt(hdlg, IDC_BLUR3, 0, FALSE);
				mfd->ifp->RedoFrame();
				break;
			case IDC_BLURBITMAP:
				mfd->blurbitmap = GetDlgItemInt(hdlg, IDC_BLURBITMAP, 0, FALSE);
				mfd->ifp->UndoSystem();
				mfd->ifp->RedoSystem();
				break;
			case IDC_WHITE:
				mfd->white = GetDlgItemInt(hdlg, IDC_WHITE, 0, FALSE);
				mfd->ifp->UndoSystem();
				mfd->ifp->RedoSystem();
				break;
			case IDC_FADEIN:
				mfd->fadein = GetDlgItemInt(hdlg, IDC_FADEIN, 0, FALSE);
				mfd->ifp->RedoFrame();
				break;
			case IDC_NOISE_LVL:
				mfd->noise_level = GetDlgItemInt(hdlg, IDC_NOISE_LVL, 0, FALSE);
				mfd->ifp->RedoFrame();
				break;
			case IDC_FEATHER:
				mfd->feather = GetDlgItemInt(hdlg, IDC_FEATHER, 0, FALSE);
				mfd->ifp->UndoSystem();
				mfd->ifp->RedoSystem();
				break;
			case IDC_RADIO1:
				CheckRadioButton(hdlg, IDC_RADIO1, IDC_RADIO5, IDC_RADIO1);
				mfd->interpolate_side = 1;
				mfd->ifp->RedoFrame();
				break;
			case IDC_RADIO2:
				CheckRadioButton(hdlg, IDC_RADIO1, IDC_RADIO5, IDC_RADIO2);
				mfd->interpolate_side = 2;
				mfd->ifp->RedoFrame();
				break;
			case IDC_RADIO3:
				CheckRadioButton(hdlg, IDC_RADIO1, IDC_RADIO5, IDC_RADIO3);
				mfd->interpolate_side = 3;
				mfd->ifp->RedoFrame();
				break;
			case IDC_RADIO4:
				CheckRadioButton(hdlg, IDC_RADIO1, IDC_RADIO5, IDC_RADIO4);
				mfd->interpolate_side = 4;
				mfd->ifp->RedoFrame();
				break;
			case IDC_RADIO5:
				CheckRadioButton(hdlg, IDC_RADIO1, IDC_RADIO5, IDC_RADIO5);
				mfd->interpolate_side = 0;
				mfd->ifp->RedoFrame();
				break;
			case IDC_XPLUS:
				if (mfd->x < 800)
				{
					mfd->x += 1;
					SetDlgItemInt(hdlg, IDC_XOFF, mfd->x, FALSE);
					SendMessage(GetDlgItem(hdlg, IDC_XSLIDER), TBM_SETPOS, (WPARAM)TRUE, mfd->x);
					mfd->ifp->RedoFrame();
				}
				break;
			case IDC_XMINUS:
				if (mfd->x > 0)
				{
					mfd->x -= 1;
					SetDlgItemInt(hdlg, IDC_XOFF, mfd->x, FALSE);
					SendMessage(GetDlgItem(hdlg, IDC_XSLIDER), TBM_SETPOS, (WPARAM)TRUE, mfd->x);
					mfd->ifp->RedoFrame();
				}
				break;
			case IDC_YPLUS:
				if (mfd->y < 600)
				{
					mfd->y += 1;
					SetDlgItemInt(hdlg, IDC_YOFF, mfd->y, FALSE);
					SendMessage(GetDlgItem(hdlg, IDC_YSLIDER), TBM_SETPOS, (WPARAM)TRUE, mfd->y);
					mfd->ifp->RedoFrame();
				}
				break;
			case IDC_YMINUS:
				if (mfd->y > 0)
				{
					mfd->y -= 1;
					SetDlgItemInt(hdlg, IDC_YOFF, mfd->y, FALSE);
					SendMessage(GetDlgItem(hdlg, IDC_YSLIDER), TBM_SETPOS, (WPARAM)TRUE, mfd->y);
					mfd->ifp->RedoFrame();
				}
				break;
			case IDC_ZERO:
				mfd->blur1 = mfd->blur2 = 0;
				mfd->blur3 = 0;
				mfd->blurbitmap = 0;
				SetDlgItemInt(hdlg, IDC_BLUR2, mfd->blur2, FALSE);
				SetDlgItemInt(hdlg, IDC_BLUR3, mfd->blur3, FALSE);
				SetDlgItemInt(hdlg, IDC_BLURBITMAP, mfd->blurbitmap, FALSE);
				mfd->ifp->UndoSystem();
				mfd->ifp->RedoSystem();
				break;
			case IDC_DEFAULTS:
				mfd->blur1 = DEFAULT_B1;
				mfd->blur2 = DEFAULT_B2;
				mfd->blur3 = DEFAULT_B3;
				mfd->blurbitmap = DEFAULT_BB;
				mfd->threshold1 = DEFAULT_T1;
				mfd->threshold2 = DEFAULT_T2;
				mfd->threshold3 = DEFAULT_T3;
				mfd->feather = DEFAULT_FEATHER;
				SetDlgItemInt(hdlg, IDC_BLUR2, mfd->blur2, FALSE);
				SetDlgItemInt(hdlg, IDC_BLUR3, mfd->blur3, FALSE);
				SetDlgItemInt(hdlg, IDC_BLURBITMAP, mfd->blurbitmap, FALSE);
				SetDlgItemInt(hdlg, IDC_THRESHOLD1, mfd->threshold1, FALSE);
				SetDlgItemInt(hdlg, IDC_THRESHOLD2, mfd->threshold2, FALSE);
				SetDlgItemInt(hdlg, IDC_THRESHOLD3, mfd->threshold3, FALSE);
				SetDlgItemInt(hdlg, IDC_FEATHER, mfd->feather, FALSE);
				mfd->ifp->UndoSystem();
				mfd->ifp->RedoSystem();

				break;
			case IDCANCEL:
				EndDialog(hdlg, 1);
				return TRUE;
			}
			break;
	}

	return FALSE;
}

int ConfigProc(FilterActivation *fa, const FilterFunctions *ff, HWND hwnd)
{
	MyFilterData *mfd = (MyFilterData *)fa->filter_data;
	MyFilterData mfd_old = *mfd;
	int ret;

	mfd->ifp = fa->ifp;

	if (DialogBoxParam(fa->filter->module->hInstModule, MAKEINTRESOURCE(IDD_FILTER),
		hwnd, ConfigDlgProc, (LPARAM)mfd))
	{
		*mfd = mfd_old;
		ret = TRUE;
	}
    else
	{
		ret = FALSE;
	}
	return(ret);
}

void StringProc(const FilterActivation *fa, const FilterFunctions *ff, char *str)
{
	MyFilterData *mfd = (MyFilterData *)fa->filter_data;
}

void BlurArea(MyFilterData *mfd, const FilterActivation *fa, struct DecomposedRGB *buf1, struct DecomposedRGB *buf2, int *blur_table, int passes) {
	unsigned long r, g, b;
	int x, y;
	const PixDim	width = fa->src.w;
	const PixDim	height = fa->src.h;
	Pixel32 *dst, *dst_fixed;
	dst_fixed = fa->dst.data;
	long dst_pitch = fa->dst.pitch;
	int X = mfd->x;
	int Y = mfd->y;
	int bheight = mfd->bheight;
	int bwidth = mfd->bwidth;
	int bpitch = mfd->bpitch;
	int i;
	int by; //absolute bitmap y coord
	int bx; //absolute bitmap x coord
	
	y = height - Y - bheight; // absolute frame y coord
	dst	= (Pixel *)((char *)dst_fixed + y * dst_pitch);
	for (by = 0; by < bheight; by++) {
		for (x = X, bx = 0; x < X + bwidth; x++, bx++)	{
			i = bx+by*bwidth;
			buf2[i].r = ((dst[x] & 0xff0000) >> 16)<<16;
			buf2[i].g = ((dst[x] & 0xff00) >> 8)<<16;
			buf2[i].b = (dst[x] & 0xff)<<16;
		}
		dst	= (Pixel *)((char *)dst + dst_pitch);
		y++;
	}
	while(passes) {
		//copy buffer 2 to buffer 1
		for (int i = 0; i < bwidth*bheight; i++) {
			buf1[i].r = buf2[i].r;
			buf1[i].g = buf2[i].g;
			buf1[i].b = buf2[i].b;
		}
		//blur from buffer 1 to buffer 2
		for (by = 1; by < bheight-1; by++) {
			for (bx = 1; bx < bwidth-1; bx++)	{
				i = bx+by*bwidth;
				if(blur_table[i]>=passes) {  
/*					buf2[i].r = (buf1[i].r + buf1[i-1].r + buf1[i+1].r
						+ buf1[i-bwidth].r + buf1[i+bwidth].r)/5;
					buf2[i].g = (buf1[i].g + buf1[i-1].g + buf1[i+1].g
						+ buf1[i-bwidth].g + buf1[i+bwidth].g)/5;
					buf2[i].b = (buf1[i].b + buf1[i-1].b + buf1[i+1].b
						+ buf1[i-bwidth].b + buf1[i+bwidth].b)/5;
*/
					buf2[i].r = (buf1[i-1].r + buf1[i+1].r
						+ buf1[i-bwidth].r + buf1[i+bwidth].r)/4;
					buf2[i].g = (buf1[i-1].g + buf1[i+1].g
						+ buf1[i-bwidth].g + buf1[i+bwidth].g)/4;
					buf2[i].b = (buf1[i-1].b + buf1[i+1].b
						+ buf1[i-bwidth].b + buf1[i+bwidth].b)/4;
				}
			}
		}
		passes--;
	}
	//copy from buffer 2 to dst
	y = height - Y - bheight; // absolute frame y coord
	dst	= (Pixel *)((char *)dst_fixed + (y+1) * dst_pitch);
	for (by = 1; by < bheight-1; by++) {
		for (x = X+1, bx = 1; x < X + bwidth-1; x++, bx++)	{
			i = bx+by*bwidth;
			r = buf2[i].r>>16;
			g = buf2[i].g>>16;
			b = buf2[i].b>>16;
			
			dst[x] = ((r << 16) | (g << 8) | (b));
		}
		dst	= (Pixel *)((char *)dst + dst_pitch);
		y++;
	}
	
}

int LoadBitmaps(MyFilterData *mfd)
{
	FILE *fp;
	char buf[6];
	DWORD size;
    char *p;
	char path[255];
	int bwidth, bheight;
	unsigned char *bp1,*bp2,*bp3;

	/* Load bitmap and get pointer to bitmap bits. */

	//bitmap1

	mfd->depth = 0;
	strcpy(path, mfd->filename);
	p = path + strlen(path);

	mfd->ModeTransparent=MT_NONE;
	mfd->ModeInterpolate=MI_NONE;

	if(path[strlen(path)-5]=='0') {
		//at least the first is present, if 2 is present switch this to color
		mfd->ModeTransparent=MT_WHITE; 
	} else if(path[strlen(path)-5]=='2') {
		mfd->ModeTransparent=MT_NONE;
		mfd->ModeInterpolate=MI_NORMAL;
	} else if(path[strlen(path)-5]=='3') {
		mfd->ModeTransparent=MT_NONE;
		mfd->ModeInterpolate=MI_EAT;
	} else {
		return 1;
	}
	if(mfd->ModeTransparent==MT_WHITE) {
		while (p >= path && *p != '.') p--;
		if (*p != '.') return 1;
		if ((fp = fopen(path, "rb")) == NULL) return 1;
		fread(buf, 1, 2, fp);
		if (buf[0] != 'B' || buf[1] != 'M') return 1;
		fread(&size, 1, 4, fp);
		fclose(fp);
		if (mfd->bitmap) free(mfd->bitmap);
		mfd->bitmap = (unsigned char *) malloc((size_t) size);
	    if ((fp = fopen(path, "rb")) == NULL)
			return 1;
		fread(mfd->bitmap, 1, size, fp);
		fclose(fp);

		mfd->bits = mfd->bitmap + *((DWORD *)(mfd->bitmap + 10));
		bwidth = mfd->bwidth = *((LONG *)(mfd->bitmap + 18));
		bheight = mfd->bheight = *((DWORD *)(mfd->bitmap + 22));
		mfd->bpitch = 3 * mfd->bwidth;
		if (mfd->bpitch & 0x3)
		{
			mfd->bpitch = (mfd->bpitch & 0xfffffffc) + 4;
		}
		mfd->depth = *((unsigned int *)(mfd->bitmap + 28));

		bp1 = mfd->bits;
		int back1b = mfd->back1b = *bp1++;
		int back1g = mfd->back1g = *bp1++;
		int back1r = mfd->back1r = *bp1++;
	
		// analyse bitmap for the highest difference, which gives indication
		// of how well the removal will work and therefore how much interpolation
		// to use
		int highest_difference=0;
		int currentr,currentg,currentb, x,y;

		for (y = 0; y < bheight; y++)
		{
			bp1 = mfd->bits + y * mfd->bpitch;
			for (x = 0; x < bwidth; x++)
			{
				currentb = *bp1++;
				currentg = *bp1++;
				currentr = *bp1++;
				if(currentb-back1b > highest_difference) highest_difference = currentb-back1b;	
				if(currentg-back1g > highest_difference) highest_difference = currentg-back1g;	
				if(currentb-back1r > highest_difference) highest_difference = currentr-back1r;	
	
			}
		}
		mfd->bitmap_difficulty = highest_difference;

		//now see if the 2nd bitmap is present
		path[strlen(path)-5]='1';
		if ((fp = fopen(path, "rb")) != NULL) {
			mfd->ModeTransparent = MT_COLOR;
			fread(buf, 1, 2, fp);
			if (buf[0] != 'B' || buf[1] != 'M') return 1;
			fread(&size, 1, 4, fp);
			fclose(fp);
			if (mfd->bitmap2) free(mfd->bitmap2);
			mfd->bitmap2 = (unsigned char *) malloc((size_t) size);
		    if ((fp = fopen(path, "rb")) == NULL)
				return 1;
			fread(mfd->bitmap2, 1, size, fp);
			fclose(fp);

			mfd->bits2 = mfd->bitmap2 + *((DWORD *)(mfd->bitmap2 + 10));
			bp2 = mfd->bits2;
			mfd->back2b = *bp2++;
			mfd->back2g = *bp2++;
			mfd->back2r = *bp2++;
		}
		// and check the third

		path[strlen(path)-5]='2';
		if ((fp = fopen(path, "rb")) != NULL) {
			mfd->ModeInterpolate = MI_NORMAL;
			fread(buf, 1, 2, fp);
			if (buf[0] != 'B' || buf[1] != 'M') return 1;
			fread(&size, 1, 4, fp);
			fclose(fp);
			if (mfd->bitmap3) free(mfd->bitmap3);
			mfd->bitmap3 = (unsigned char *) malloc((size_t) size);
		    if ((fp = fopen(path, "rb")) == NULL)
				return 1;
			fread(mfd->bitmap3, 1, size, fp);
			fclose(fp);

			mfd->bits3 = mfd->bitmap3 + *((DWORD *)(mfd->bitmap3 + 10));
			bp3 = mfd->bits3;
			mfd->back3b = *bp3++;
			mfd->back3g = *bp3++;
			mfd->back3r = *bp3++;
		} else {
			path[strlen(path)-5]='3';
			if ((fp = fopen(path, "rb")) != NULL) {
				mfd->ModeInterpolate = MI_EAT;
				fread(buf, 1, 2, fp);
				if (buf[0] != 'B' || buf[1] != 'M') return 1;
				fread(&size, 1, 4, fp);
				fclose(fp);
				if (mfd->bitmap3) free(mfd->bitmap3);
				mfd->bitmap3 = (unsigned char *) malloc((size_t) size);
				if ((fp = fopen(path, "rb")) == NULL)
					return 1;
				fread(mfd->bitmap3, 1, size, fp);
				fclose(fp);

				mfd->bits3 = mfd->bitmap3 + *((DWORD *)(mfd->bitmap3 + 10));
				bp3 = mfd->bits3;
				mfd->back3b = *bp3++;
				mfd->back3g = *bp3++;
				mfd->back3r = *bp3++;
			

			} else {
				mfd->ModeInterpolate = MI_USE_1;
				mfd->bitmap3 = mfd->bitmap;
				mfd->bits3   = mfd->bits;
				mfd->back3b = mfd->back1b;
				mfd->back3g = mfd->back1g;
				mfd->back3r = mfd->back1r;
			}
		}
	} else {
		//only the interpolation bitmap is present
	
		while (p >= path && *p != '.') p--;
		if (*p != '.') return 1;
		if ((fp = fopen(path, "rb")) == NULL) return 1;
		fread(buf, 1, 2, fp);
		if (buf[0] != 'B' || buf[1] != 'M') return 1;
		fread(&size, 1, 4, fp);
		fclose(fp);
		if (mfd->bitmap3) free(mfd->bitmap3);
		mfd->bitmap3 = (unsigned char *) malloc((size_t) size);

		if(mfd->bitmap3==NULL) return 1;

	    if ((fp = fopen(path, "rb")) == NULL) return 1;
		fread(mfd->bitmap3, 1, size, fp);
		fclose(fp);
		mfd->bits3 = mfd->bitmap3 + *((DWORD *)(mfd->bitmap3 + 10));
		bwidth = mfd->bwidth = *((LONG *)(mfd->bitmap3 + 18));
		bheight = mfd->bheight = *((DWORD *)(mfd->bitmap3 + 22));
		mfd->bpitch = 3 * mfd->bwidth;
		if (mfd->bpitch & 0x3)	mfd->bpitch = (mfd->bpitch & 0xfffffffc) + 4;
		mfd->depth = *((unsigned int *)(mfd->bitmap3 + 28));

		// Get background 
		unsigned char *bp3;

		bp3 = mfd->bits3;
		mfd->back3b = *bp3++;
		mfd->back3g = *bp3++;
		mfd->back3r = *bp3++;
	}
	
	return 0;
}

int LoadInterpolationTables(MyFilterData *mfd, const FilterActivation *fa)
{
	int bwidth = mfd->bwidth;
	int bheight = mfd->bheight;
	int backr = mfd->back3r;
	int backg = mfd->back3g;
	int backb = mfd->back3b;
	int i=0;

	int currentr,currentg,currentb, x,y;
	unsigned char *bp3;

	//initialize tables
	mfd->logobuffer1 = (struct DecomposedRGB *)malloc(sizeof(DecomposedRGB)*bheight*bwidth);
	if(mfd->logobuffer1 == NULL) return 1;

	mfd->logobuffer2 = (struct DecomposedRGB *)malloc(sizeof(DecomposedRGB)*bheight*bwidth);
	if(mfd->logobuffer2 == NULL) return 1;

	mfd->logobuffer3 = (struct DecomposedRGB *)malloc(sizeof(DecomposedRGB)*bheight*bwidth);
	if(mfd->logobuffer3 == NULL) return 1;

	mfd->logobuffer_eat = (int *)malloc(sizeof(int)*bheight*bwidth);
	if(mfd->logobuffer_eat == NULL) return 1;

	mfd->Prevlogo = (Pixel32 *)malloc(sizeof(Pixel32)*bheight*bwidth);
	if(mfd->Prevlogo == NULL) return 1;

	mfd->scalar_table2 = (int*)malloc(sizeof(int)*bheight*bwidth);
	if(mfd->scalar_table2 == NULL) return 1;

	mfd->scalar_table3 = (int*)malloc(sizeof(int)*bheight*bwidth);
	if(mfd->scalar_table3 == NULL) return 1;

	mfd->scalar_table = (int*)malloc(sizeof(int)*bheight*bwidth);
	if(mfd->scalar_table == NULL) return 1;

	mfd->ns_table = (int*)malloc(sizeof(int)*bheight*bwidth);
	if(mfd->ns_table == NULL) return 1;

	mfd->ew_table = (int*)malloc(sizeof(int)*bheight*bwidth);
	if(mfd->ew_table == NULL) return 1;

	mfd->interpolation_data = (struct InterPixelData *)malloc(sizeof(InterPixelData)*bheight*bwidth);
	if(mfd->interpolation_data == NULL) return 1;

	int *scalar_table = mfd->scalar_table;

	for (y = 0; y < bheight; y++)
	{
		bp3 = mfd->bits3 + y * mfd->bpitch;
		for (x = 0; x < bwidth; x++)
		{
			currentb = *bp3++;
			currentg = *bp3++;
			currentr = *bp3++;
			if(currentb!=backb || currentg!=backg || currentr!=backr) {
				scalar_table[x+y*bwidth]=0;
			} else {
				scalar_table[x+y*bwidth]=1;
			}
		}
	}
	if(mfd->ModeInterpolate==MI_NORMAL || mfd->ModeInterpolate==MI_USE_1) {
		if(GetInterpolationPts(mfd, scalar_table, mfd->interpolation_data, fa)) return 1;
		// border tables
		for (y = 0; y < bheight; y++)
		{
			for (x = 0; x < bwidth; x++)
			{
				mfd->ew_table[x+y*bwidth]=0;
			}
		}
		for (y = 0; y < bheight; y++)
		{
			for (x = 1; x < bwidth; x++)
			{
				i = x+y*bwidth;
				if(scalar_table[i] != scalar_table[i-1]) {
					if(scalar_table[i] == 0) {
						mfd->ew_table[i]=-1;
					} else {
						mfd->ew_table[i-1]=1;
					}
				}
			}
		}
		for (y = 0; y < bheight; y++)
		{
			for (x = 0; x < bwidth; x++)
			{
				mfd->ns_table[x+y*bwidth]=0;
			}
		}
		for (x = 0; x < bwidth; x++)
		{
			for (y = 1; y < bheight; y++)
			{
				i = x+y*bwidth;
				if(scalar_table[i] != mfd->scalar_table[i-bwidth]) {
					if(scalar_table[i] == 0) {
						mfd->ns_table[i]=-1;
					} else {
						mfd->ns_table[i-bwidth]=1;
					}
				}
			}
		}

		mfd->NumPoints = 0;
		for(i=0; i<bwidth*bheight; i++) {
			if(mfd->ns_table[i] || mfd->ew_table[i]) mfd->NumPoints++;
		}
	}
	if(mfd->ModeInterpolate==MI_EAT) {
		mfd->NumPoints = 0;
		for (y = 0; y < bheight; y++)
		{
			bp3 = mfd->bits3 + y * mfd->bpitch;
			for (x = 0; x < bwidth; x++)
			{
				i = x+y*bwidth;
				currentb = *bp3++;
				currentg = *bp3++;
				currentr = *bp3++;
				if(currentb!=0 || currentg!=0 || currentr!=255) {
					mfd->logobuffer_eat[i]=1;
					scalar_table[i]=1;
					mfd->NumPoints++;
				} else if(currentb!=0 || currentg!=255 || currentr!=0){
					mfd->logobuffer_eat[i]=2;
					scalar_table[i]=1;
					mfd->NumPoints++;
				} else {
					mfd->logobuffer_eat[i]=0;
					scalar_table[i]=0;
				}
			}
		}

	}

	mfd->PrevFrameBorder = (struct DecomposedRGB *) malloc(sizeof(DecomposedRGB)*mfd->NumPoints);
	if(mfd->PrevFrameBorder==NULL) return 1;

	for(i=0; i<mfd->NumPoints; i++) {
		mfd->PrevFrameBorder[i].r = 0;
		mfd->PrevFrameBorder[i].g = 0;
		mfd->PrevFrameBorder[i].b = 0;
	}
	mfd->PrevWeight = 1;

	int feather = mfd->feather;
	int feather_temp;
	int blurbitmap = mfd->blurbitmap;
	// set up bitmap blur table with feathering
	for(i=0; i < bheight*bwidth; i++) {
		mfd->scalar_table3[i] = mfd->blurbitmap;
		mfd->Prevlogo[i] = 0;
	}
	if(feather) {
		int z;
		feather_temp = blurbitmap/feather;
		for(z=0,feather_temp = blurbitmap/feather; z<feather; z++,feather_temp+=blurbitmap/feather) {
			y=z;
			for(x=z; x<bwidth-z; x++) {
				i = x+y*bwidth;
				mfd->scalar_table3[i] = feather_temp;
			}
			y=bheight-z-1;
			for(x=z; x<bwidth-z; x++) {
				i = x+y*bwidth;
				mfd->scalar_table3[i] = feather_temp;
			}
			x=z;
			for(y=z; y<bheight-z; y++) {
				i = x+y*bwidth;
				mfd->scalar_table3[i] = feather_temp;
			}
			x=bwidth-z-1;
			for(y=z; y<bheight-z; y++) {
				i = x+y*bwidth;
				mfd->scalar_table3[i] = feather_temp;
			}
		}
	}


	// set up lookup table for white mode
	if(mfd->ModeTransparent==MT_WHITE) {
		double white= (double)mfd->white;
		double black = (double)mfd->back1r;
		int max = 255;
		int min = 0;
		double slope,intercept;
		int src, logo, dst;
		for(src=0; src<256; src++) {
			for(logo=0; logo<256; logo++) {
				if(white-logo) {
					slope = (white-black)/(white-logo);
				} else {
					slope = 0;
				}
				intercept = black - (slope * logo);
				dst = (int)(slope*src+intercept);

				if (dst < min) dst = min;
				if (dst > max) dst = max;
	
				mfd->conversiontable[src][logo] = dst;
			}
		}
	}


	return 0;
}


int GetInterpolationPts(MyFilterData *mfd, int* table, struct InterPixelData *data, const FilterActivation *fa) {
	int x,y;  // local coords
	int bheight = mfd->bheight;
	int bwidth  = mfd->bwidth;
	int height = fa->src.h;
	int X = mfd->x;
	int Y = mfd->y;

	int wx, wy = height - Y +1 - bheight;   // absolute coords

	for (y = 1; y < bheight-1; y++, wy++) {
		for (x = 1, wx = X+1; x < bwidth-1; x++, wx++) {
			int i = x+y*bwidth;
			if(table[i]==0) {
				//find the nearest background pt in each NESW direction
				int dx,dy,cval,cd,highd=0;
				//north
				dy = 0;
				dx = 0;
				cval=0;
				while(cval==0) {
					dy--;
					if(y+dy<0) return 1;
					cval = table[x+(y+dy)*bwidth];
				}
				cd = data[i].nn.d = abs(dy);
				data[i].nn.offset = (wx + dx)*4 + (wy + dy) * fa->src.pitch;
				
				//east
				dy = 0;
				dx = 0;
				cval=0;
				while(cval==0) {
					dx++;
					if(x+dx>bwidth) return 1;
					cval = table[x+dx+y*bwidth];
				}
				cd = data[i].ee.d = dx;
				data[i].ee.offset = (wx + dx)*4 + (wy + dy) * fa->src.pitch;
				
				//south
				dy = 0;
				dx = 0;
				cval=0;
				while(cval==0) {
					dy++;
					if(y+dy>bheight) return 1;
					cval = table[x+(y+dy)*bwidth];
				}
				cd = data[i].ss.d = dy;
				data[i].ss.offset = (wx + dx)*4 + (wy + dy) * fa->src.pitch;
				
				//west
				dy = 0;
				dx = 0;
				cval=0;
				while(cval==0) {
					dx--;
					if(x+dx<0) return 1;
					cval = table[x+dx+y*bwidth];
				}
				cd = data[i].ww.d = abs(dx);
				data[i].ww.offset = (wx + dx)*4 + (wy + dy) * fa->src.pitch;
				
			}
		}
	}
	return 0;
}


bool FssProc(FilterActivation *fa, const FilterFunctions *ff, char *buf, int buflen)
{
	MyFilterData *mfd = (MyFilterData *)fa->filter_data;
    char save[255];
	int i, j;

	i = j = 0;
	while(mfd->filename[i])
	{
		if (mfd->filename[i] == '\\')
		{
			save[j++] = '\\';
			save[j++] = '\\';
			i++;
		}
		else
		{
			save[j++] = mfd->filename[i++];
		}
	}
	save[j] = 0;

	unsigned int thresholds = ((mfd->threshold1 << 16) | (mfd->threshold2 << 8) | (mfd->threshold3));
	unsigned int blurs = ((mfd->blur2 << 16) | (mfd->blur3 << 8) | (mfd->blurbitmap));

	_snprintf(buf, buflen, "Config(\"%s\",%d, %d, %d, %d, %d, %d, %d, %d, %d, %d, %d, %d, %d, %d)",
		save,
		mfd->ModeTransparent,
		mfd->ModeInterpolate,
		mfd->x,
		mfd->y,
		mfd->start,
		mfd->duration,
		mfd->fadein,
		mfd->fadeout,
		mfd->alpha,
		thresholds,
		blurs,
		mfd->white, 
		mfd->interpolate_side,
		mfd->ScenePercent
		);		//1 free

	return true;
}

void ScriptConfig(IScriptInterpreter *isi, void *lpVoid, CScriptValue *argv, int argc) {
	FilterActivation *fa = (FilterActivation *)lpVoid;
	MyFilterData *mfd = (MyFilterData *)fa->filter_data;
	
	unsigned int thresholds;
	unsigned int blurs;

	strcpy(mfd->filename,	 *argv[0].asString());

	LoadBitmaps(mfd);	// this is required only to get proper info displayed
						// in the config screen before processing starts

	mfd->ModeTransparent	= argv[1].asInt();
	mfd->ModeInterpolate	= argv[2].asInt();
	mfd->x					= argv[3].asInt();
	mfd->y					= argv[4].asInt();
	mfd->start				= argv[5].asInt();
	mfd->duration			= argv[6].asInt();
	mfd->fadein				= argv[7].asInt();
	mfd->fadeout			= argv[8].asInt();
	mfd->alpha				= argv[9].asInt();
	thresholds				= argv[10].asInt();
	blurs					= argv[11].asInt();
	mfd->white				= argv[12].asInt();
	mfd->interpolate_side	= argv[13].asInt();
	mfd->ScenePercent		= argv[14].asInt();

	mfd->threshold1 = ((thresholds & 0xff0000) >> 16);
	mfd->threshold2 = ((thresholds & 0xff00) >> 8);
	mfd->threshold3 = (thresholds & 0xff);

	mfd->blur2 = ((blurs & 0xff0000) >> 16);
	mfd->blur3 = ((blurs & 0xff00) >> 8);
	mfd->blurbitmap = (blurs & 0xff);

}

